@@ -15,14 +15,17 @@ from shinken.subprocess_helper.error_handler import ERROR_LEVEL, ErrorHandler
 from .compat import Empty as EmptyQueue
 from .log import logger as _logger, LoggerFactory, PART_INITIALISATION, PartLogger
 from .runtime_stats.threads_dumper import thread_dumper
+from .runtime_stats.memory_stats import MemoryStats
 from .thread_helper import Thread
 from .util import start_malloc_trim_thread, set_process_name
 
 if TYPE_CHECKING:
-    from .misc.type_hint import List, Any, Optional, Callable, Str, Number, Union
+    from .ipc.shinken_queue.shinken_queue import ShinkenQueue
     from .log import PartLogger
+    from .misc.type_hint import List, Any, Optional, Callable, Str, Number, Union
 
 ARG_SEPARATOR = ':::'
+# noinspection SpellCheckingInspection
 _UNINTERRUPTABLE_SLEEP_TIME = 1.0
 _old_process = {}
 
@@ -60,6 +63,7 @@ class LookAtMyFatherThread(Thread):
             os.kill(self.father_pid, 0)
         except:
             self.logger.error('The father process [%s]-[%s] is dead, I kill [%s]-[%s].' % (self.father_pid, self.father_name, os.getpid(), self.to_kill_name))
+            # noinspection PyUnresolvedReferences, PyProtectedMember
             os._exit(0)
     
     
@@ -117,13 +121,18 @@ class EventHandler(Thread):
 
 class QueueHandler(Thread):
     
-    def __init__(self, max_size, queue_name=u'queue-handler', error_handler=None):
+    def __init__(self, max_size, queue_name=u'queue-handler', error_handler=None, item_keep_duration=0):
         super(QueueHandler, self).__init__(error_handler=error_handler)
         self.owner_pid = os.getpid()
         self._queue = Queue()
         self._queue_name = queue_name
         self._max_size = max_size
         self.all_items = []
+        self.item_keep_duration = item_keep_duration
+    
+    
+    def init(self):
+        self._queue = Queue()
     
     
     def stop(self):
@@ -154,8 +163,15 @@ class QueueHandler(Thread):
         try:
             while True:
                 item = self._queue.get_nowait()
-                self.all_items.append(item)
+                now = time.time()
+                self.all_items.append((now, item))
                 self.on_add_item(item)
+                if self.all_items and self.item_keep_duration:
+                    for i in self.all_items:
+                        if (now - i[0]) > self.item_keep_duration:
+                            self.all_items.pop(0)
+                        else:
+                            break
                 if len(self.all_items) > self._max_size:
                     self.all_items.pop(0)
                 self.interruptable_sleep(self.loop_speed)
@@ -238,7 +254,7 @@ class FromModuleCommandRespond(object):
 
 class CommandQueueHandler(Thread):
     def __init__(self, name, parent_logger, commands_to_q, commands_from_q, main_process):
-        # type: (Str, PartLogger, Callable, Callable, object) -> None
+        # type: (Str, PartLogger, ShinkenQueue, ShinkenQueue, object) -> None
         super(CommandQueueHandler, self).__init__(loop_speed=-1, logger=parent_logger)
         self.daemon = True
         self.logger = parent_logger
@@ -305,7 +321,7 @@ class CommandQueueHandler(Thread):
                 retry_count = 0
                 before = time.time()
                 self.commands_to_q.put(request)
-                while True:  # will timeout at max 2s because the commands request are block and we are the only one asking for it
+                while True:  # will time out at max 2s because the commands request are blocked, and we are the only one asking for it
                     try:
                         response = self.commands_from_q.get(block=True, timeout=timeout)  # type: FromModuleCommandRespond
                     except EmptyQueue:
@@ -339,7 +355,7 @@ class CommandQueueHandler(Thread):
 
 
 class BaseSubProcess(object):
-    def __init__(self, name, father_name='UNSET', loop_speed=1, stop_process_on_error=True, only_one_process_by_class=False, error_handler=None):
+    def __init__(self, name, father_name=u'UNSET', loop_speed=1, stop_process_on_error=True, only_one_process_by_class=False, error_handler=None):
         # type: (unicode, unicode, int, bool, bool, Optional[ErrorHandler]) -> None
         my_class = type(self)
         old_process = get_old_process(my_class)
@@ -362,6 +378,9 @@ class BaseSubProcess(object):
         self.look_at_father_thread = LookAtMyFatherThread(self.father_pid, self.father_name, self.name, logger=self.logger)
         
         register_old_process(my_class, self)
+        
+        if TYPE_CHECKING:
+            self.loaded_into = ''
     
     
     def update_father(self, father_name):
@@ -376,23 +395,7 @@ class BaseSubProcess(object):
         if not ON_LINUX:
             return
         if sig == signal.SIGUSR1:  # if USR1, ask a memory dump
-            if sys.version_info[1] == 6:  # python 2.6
-                try:
-                    from guppy import hpy
-                    hp = hpy()
-                    self.logger.error('(support-only) MEMORY DUMP (to be sent to the support):\n%s' % hp.heap())
-                    return
-                except ImportError:
-                    self.logger.error('(support-only) MEMORY DUMP: FAIL check if guppy lib is installed')
-            if sys.version_info[1] == 7:  # python 2.7
-                try:
-                    import meliae.scanner
-                    import meliae.loader
-                    _f = "/tmp/memorydump-%s.json" % self.name
-                    meliae.scanner.dump_all_objects(_f)
-                    self.logger.error('(support-only) Memory information dumped to file %s (to be sent to the support)' % _f)
-                except ImportError:
-                    self.logger.error('(support-only) MEMORY DUMP: FAIL check if meliae lib is installed')
+            MemoryStats.dump_memory_full_memory_dump(self.name)
         elif sig == signal.SIGPWR:  # SIGPWR (old signal not used) dump all threads stacks
             thread_dumper.dump_all_threads()
         else:
@@ -414,14 +417,13 @@ class BaseSubProcess(object):
     
     
     # In the sub process, we should start some threads, like the one that look at parent
-    # etc etc. Sub Class can have other threads for special purpose
+    # etc. etc. Sub Class can have other threads for special purpose
     def _start_sub_process_threads(self):
         start_malloc_trim_thread()
         self.look_at_father_thread.start_thread()
     
     
     def _set_logger_name(self):
-        global logger
         logger_name = self.get_logger_name()
         if isinstance(logger_name, (list, tuple)):
             logger_names = [i.replace('[', '').replace(']', '') for i in logger_name]
@@ -462,14 +464,14 @@ class BaseSubProcess(object):
         # Our sub process need some utility threads, like:
         # * malloc trimming
         # * looking at father process (if dead, we die)
-        # * others set by the sub class
+        # * others set by the subclass
         self._start_sub_process_threads()
         logger_init.info('Sub process ready to work.')
     
     
     def get_process_name(self):
-        # type: () -> str
-        return '%s [ %s ]' % (self.father_name, self.name)
+        # type: () -> unicode
+        return u'%s [ %s ]' % (self.father_name, self.name)
     
     
     def get_logger_name(self):
@@ -509,7 +511,7 @@ class BaseSubProcess(object):
     
     
     def interruptable_sleep(self, raw_duration):
-        # type: (float) -> NoReturn
+        # type: (Number) -> None
         if raw_duration <= _UNINTERRUPTABLE_SLEEP_TIME:
             time.sleep(raw_duration)
             return
@@ -569,6 +571,10 @@ class BaseSubProcess(object):
         self._process = None
     
     
+    def is_alive(self):
+        return self._process and self._process.is_alive()
+    
+    
     def _kill(self):
         if self._process is None:
             return
