@@ -24,10 +24,10 @@
 
 import base64
 import cPickle
+import logging
 import os
 import threading
 import time
-import traceback
 import zlib
 from multiprocessing import active_children
 
@@ -594,8 +594,6 @@ class Receiver(WithInventorySatellite):
             
             self.load_modules_manager()
             
-            self._register_http_interfaces()
-            
             #  We wait for initial conf
             self.wait_for_initial_conf()
             if not self.new_conf:
@@ -607,7 +605,7 @@ class Receiver(WithInventorySatellite):
             self.do_mainloop()
         
         except Exception:
-            logger.critical('The daemon did have an unrecoverable error. It must exit.')
-            logger.critical('You can log a bug to your Shinken integrator with the error message:')
-            logger.critical('%s' % (traceback.format_exc()))
+            logger.critical(u'The daemon did have an unrecoverable error. It must exit.')
+            logger.critical(u'You can log a bug to your Shinken integrator with the error message:')
+            logger.print_stack(level=logging.CRITICAL)
             raise
