@@ -24,6 +24,7 @@
 
 import codecs
 import os
+import re
 from ctypes import c_int
 from logging.handlers import TimedRotatingFileHandler
 from multiprocessing import RLock, Value
@@ -106,6 +107,7 @@ LOG_SECTION_SIZE = 22
 LONG_FLUSH_LIMIT = 1  # if flush is longer than 1s, warn about it
 LONG_FLUSH_KEEP_DURATION = 60  # keep the long flush during a 1min delay
 DEFAULT_LONG_FLUSH_STATS = {'is_too_long': False, 'write_duration': 0.0, 'log_path': ''}
+TAG_RE = re.compile(r'<[^>]+>')
 
 
 def get_chapter_string(chapter):
@@ -381,7 +383,7 @@ class Log(logging.Logger):
         self.do_log_parameters = do_log_parameters
         self.set_human_format(on=True)  # activate by default human format
         self.log_file_path = None
-        self.loggers_info = None
+        self.loggers_info = None  # Optional[LoggersInfo]
         self._my_daemon = None
     
     
@@ -617,25 +619,33 @@ def naglog_result(level, message, *args):  # noqa => we know aout *args
 
 
 class LoggersInfo(ShareItem):
-    FULL_NAME_PREFIX = 'full_name_____'
-    ENABLE_PREFIX = 'enable_____'
+    FULL_NAME_PREFIX = u'full_name_____'
+    ENABLE_PREFIX = u'enable_____'
     
     
     def set_logger_full_name(self, loggers_info_id, full_name):
-        # type: (str, unicode) -> NoReturn
-        setattr(self, '%s%s' % (LoggersInfo.FULL_NAME_PREFIX, loggers_info_id), full_name)
+        # type: (unicode, unicode) -> NoReturn
+        id_ = u'%s%s' % (LoggersInfo.FULL_NAME_PREFIX, loggers_info_id)
+        setattr(self, id_, full_name)
     
     
     def set_logger_enable(self, loggers_info_id, enable):
-        # type: (str, bool) -> None
-        for key_name in self.get_all_attr_read_only().iterkeys():
-            if key_name.startswith('%s%s' % (LoggersInfo.ENABLE_PREFIX, loggers_info_id)):
-                setattr(self, key_name, enable)
-        setattr(self, '%s%s' % (LoggersInfo.ENABLE_PREFIX, loggers_info_id), enable)
+        # type: (unicode, bool) -> None
+        all_infos = self.get_all_attr_read_only()
+        logger_id = u'%s%s' % (LoggersInfo.FULL_NAME_PREFIX, loggers_info_id)
+        logger_parts = all_infos.get(logger_id, None)
+        if logger_parts is None:
+            return
+        parts_len = len(logger_parts)
+        for key_name, parts in all_infos.iteritems():
+            if not key_name.startswith(LoggersInfo.FULL_NAME_PREFIX) or not isinstance(parts, list) or logger_parts != parts[:parts_len]:
+                continue
+            enable_key = u'%s%s' % (LoggersInfo.ENABLE_PREFIX, key_name.replace(LoggersInfo.FULL_NAME_PREFIX, u''))
+            setattr(self, enable_key, enable)
     
     
     def get_logger_enable(self, loggers_info_id):
-        # type: (str) -> bool
+        # type: (unicode) -> bool
         attr_name = u'%s%s' % (LoggersInfo.ENABLE_PREFIX, loggers_info_id)
         return getattr(self, attr_name.encode(u'ascii', u'ignore'), True)
     
@@ -648,7 +658,7 @@ class LoggersInfo(ShareItem):
                 loggers_info_id = key_name[len(LoggersInfo.FULL_NAME_PREFIX):]
                 info[loggers_info_id] = info.get(loggers_info_id, {})
                 info[loggers_info_id][u'name'] = value
-                info[loggers_info_id][u'display_name'] = ' '.join(('[%s]' % j for j in value if j))
+                info[loggers_info_id][u'display_name'] = u' '.join((u'[%s]' % j for j in value if j))
                 info[loggers_info_id][u'id'] = loggers_info_id
             if key_name.startswith(LoggersInfo.ENABLE_PREFIX):
                 loggers_info_id = key_name[len(LoggersInfo.ENABLE_PREFIX):]
@@ -672,8 +682,8 @@ class PartLogger(object):
         # type: (unicode, int, List[Tuple[unicode, int]], Optional[List[unicode]], int,bool) -> NoReturn
         self.name = name
         self.full_name = [logger.name.strip(), name] if full_name is None else full_name[:] + [name]
-        self.loggers_info_id = '__'.join([i.replace(' ', '_') for i in self.full_name])
         self.register = register
+        self.loggers_info_id = self._get_loggers_info_id()
         self._global_register()
         self.level = level
         self.part_name_size = part_name_size
@@ -702,6 +712,14 @@ class PartLogger(object):
         self._global_register()
     
     
+    def _get_loggers_info_id(self):
+        # type:() -> unicode
+        if six.PY2:
+            return hex(hash(u'__'.join([i.replace(u' ', u'_') for i in self.full_name])) + sys.maxint).replace(u'0x', u'log_id_').decode(u'utf-8')
+        else:
+            return hex(hash(u'__'.join([i.replace(u' ', u'_') for i in self.full_name])) + sys.maxsize).replace(u'0x', u'log_id_')
+    
+    
     def _can_global_register(self):
         if not self.register:
             return False
@@ -740,9 +758,6 @@ class PartLogger(object):
         if part_name_size is None:
             part_name_size = len(sub_part)
         
-        if part_name_size is None:
-            part_name_size = len(sub_part)
-        
         sub_part_log = PartLogger(sub_part, self.level + 1, self.display_parts, self.full_name, part_name_size, self.register and register)
         self.sub_part_logger[sub_part] = sub_part_log
         return sub_part_log
@@ -800,6 +815,12 @@ class PartLogger(object):
         logger.print_stack(prefix='%s %s' % (self._parts_string, prefix), level=level)
     
     
+    @staticmethod
+    def unescape_html(message):
+        # type: (unicode) -> unicode
+        return TAG_RE.sub(u'', message)
+    
+    
     @staticmethod
     def format_sla_date(sla_date):
         try:
@@ -808,10 +829,18 @@ class PartLogger(object):
             return '%s' % sla_date
     
     
+    @staticmethod
+    def iso_format_sla_date(sla_date):
+        try:
+            return datetime.strptime('%s %s 00:00:00' % sla_date, '%j %Y %H:%M:%S').strftime('%Y-%m-%d')
+        except:
+            return '%s' % sla_date
+    
+    
     @staticmethod
     def format_time(_time):
         try:
-            return datetime.fromtimestamp(_time).strftime('%d-%m-%Y %H:%M:%S')
+            return datetime.fromtimestamp(_time).strftime('%Y-%m-%d %H:%M:%S')
         except:
             return '%s' % _time
     
@@ -827,7 +856,7 @@ class PartLogger(object):
     @staticmethod
     def format_time_with_micro(_time):
         try:
-            return datetime.fromtimestamp(_time).strftime('%d-%m-%Y %H:%M:%S:%f')
+            return datetime.fromtimestamp(_time).strftime('%Y-%m-%d %H:%M:%S:%f')
         except:
             return '%s' % _time
     
@@ -907,6 +936,8 @@ class PartLogger(object):
 
 
 PART_INITIALISATION = u'INITIALISATION'
+PART_LOADING_CONFIGURATION = u'LOADING CONFIGURATION'
+PART_UNAVAILABILITY = u'UNAVAILABILITY'
 DEFAULT_LOG = u''
 loggers = {DEFAULT_LOG: PartLogger(DEFAULT_LOG)}
 
@@ -916,6 +947,9 @@ class LoggerFactory(object):
     @classmethod
     def get_logger(cls, name=DEFAULT_LOG, part_name_size=LOG_CHAPTER_SIZE):
         # type: (unicode, int) -> PartLogger
+        if six.PY2 and not isinstance(name, basestring):
+            name = unicode(name)
+        
         if not name:
             return loggers[DEFAULT_LOG]
         
@@ -928,3 +962,19 @@ class LoggerFactory(object):
             loggers[name] = log
         
         return log
+    
+    
+    @classmethod
+    def get_loading_configuration_logger(cls):
+        return LoggerFactory.get_logger(PART_LOADING_CONFIGURATION)
+    
+    
+    @classmethod
+    def get_unavailability_logger(cls):
+        return LoggerFactory.get_logger(PART_UNAVAILABILITY)
+    
+    
+    @classmethod
+    def after_fork_cleanup(cls):
+        global loggers
+        loggers = {DEFAULT_LOG: PartLogger(DEFAULT_LOG)}
