@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
-
-# Copyright (C) 2009-2012:
+#
+# Copyright (C) 2009-2022:
 #    Gabes Jean, naparuba@gmail.com
 #    Gerhard Lausser, Gerhard.Lausser@consol.de
 #    Gregory Starck, g.starck@gmail.com
@@ -21,30 +21,33 @@
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with Shinken.  If not, see <http://www.gnu.org/licenses/>.
+
 import itertools
 import random
 import threading
 import time
+from weakref import proxy as weak_ref, ProxyType as WeakRef
 
-# Import all objects we will need
 from shinken.brok import Brok, BROK_VERSION_V020801P15
 from shinken.brokerlink import BrokerLink, BrokerLinks
 from shinken.check import NO_END_VALIDITY
 from shinken.configuration_incarnation import PartConfigurationIncarnationContainer, PartConfigurationIncarnation
 from shinken.log import logger, get_chapter_string, LoggerFactory
 from shinken.message import Message
-from shinken.misc.filter import init_user_item_access_cache, get_element_full_name
+from shinken.misc.filter import clear_user_access_cache, init_user_item_access_cache, get_element_full_name
 from shinken.misc.type_hint import TYPE_CHECKING
+from shinken.objects.broker.broker_host import BrokerHost
+from shinken.objects.broker.broker_service import BrokerService
 from shinken.objects.command import Command, Commands
 from shinken.objects.config import Config
 from shinken.objects.contact import Contact, Contacts
 from shinken.objects.contactgroup import Contactgroup, Contactgroups
-from shinken.objects.host import Host, Hosts
+from shinken.objects.host import Hosts
 from shinken.objects.host import VIEW_CONTACTS_DEFAULT_VALUE
 from shinken.objects.hostgroup import Hostgroup, Hostgroups
 from shinken.objects.notificationway import NotificationWay, NotificationWays
 from shinken.objects.proxyitem import proxyitemsgraph
-from shinken.objects.service import Service, Services
+from shinken.objects.service import Services
 from shinken.objects.servicegroup import Servicegroup, Servicegroups
 from shinken.objects.state_id import STATE_ID, STATE_TYPE_ID
 from shinken.objects.timeperiod import Timeperiod, Timeperiods
@@ -54,7 +57,8 @@ from shinken.receiverlink import ReceiverLink, ReceiverLinks
 from shinken.schedulerlink import SchedulerLink, SchedulerLinks
 from shinken.thread_helper import Thread
 from shinken.toolbox.full_status_manipulator import FullStatusManipulator
-from shinken.util import force_memory_trimming, memory_trimming_disable_gc_collect_2, memory_trimming_allow_some_calls_to_gc_collect_2
+from shinken.util import memory_trimming_disable_gc_collect_2, memory_trimming_allow_some_calls_to_gc_collect_2
+from shinkensolutions.broker.monitoring_item_manager.filter import StaticCache
 from shinkensolutions.indexed_collection.collection import IndexedCollection
 
 if TYPE_CHECKING:
@@ -174,6 +178,10 @@ class Regenerator(object):
         # Indexed data
         self.with_indexed = with_indexed_items
         self.indexed_items = IndexedCollection()
+        self.indexed_items_2 = {}
+        self.indexed_items.add_index(u'contact_name', self.get_contact_names, on_list=True)
+        self.indexed_items.add_index(u'type', Regenerator.get_type)
+        self.indexed_items.add_index(u'realm', lambda _h: _h.get_realm())
         
         # And in progress one
         self.inp_hosts = {}
@@ -202,6 +210,8 @@ class Regenerator(object):
         self.freshness_thread = None  # type: Optional[RegeneratorFreshnessThread]
         self.freshness_lock = threading.Condition(threading.RLock())
         
+        self.last_configuration_handle_uuid = None
+        self.time_start_load_new_configuration = None
         self.part_configuration_incarnation_container = PartConfigurationIncarnationContainer()
         
         # Know in which module we are loaded
@@ -212,7 +222,20 @@ class Regenerator(object):
         
         # Flags
         self._have_received_initial_broks = False
-        self._detail_timer = {}
+        self.detail_timer = {}
+        self.item_counter = {
+            u'hosts'           : 0,
+            u'clusters'        : 0,
+            u'services'        : 0,
+            u'contacts'        : 0,
+            u'notificationways': 0,
+            u'hostgroups'      : 0,
+            u'servicegroups'   : 0,
+            u'contactgroups'   : 0,
+            u'timeperiods'     : 0,
+            u'commands'        : 0,
+            u'realms'          : 0,
+        }
     
     
     @staticmethod
@@ -293,7 +316,8 @@ class Regenerator(object):
         
         # Invalidation of list cache (see webui-enterprise/plugins/tabular/tabular.py test_remote_tabular)
         if hasattr(element, 'list_cache_var'):
-            delattr(element, 'list_cache_var')
+            # delattr(element, 'list_cache_var')
+            element.list_cache_var = {}
     
     
     # We have a thread that is doing freshness checks and modification if need
@@ -327,7 +351,6 @@ class Regenerator(object):
         try:
             self._all_done_linking(part_configuration_incarnation)
         finally:
-            force_memory_trimming()
             memory_trimming_allow_some_calls_to_gc_collect_2(2)
     
     
@@ -497,9 +520,10 @@ class Regenerator(object):
             
             # Now link with host
             host_name = service.host_name
-            service.host = self.hosts.find_by_name(host_name)
+            service_host = self.hosts.find_by_name(host_name)
+            service.host = weak_ref(service_host) if service_host else service_host
             if service.host:
-                service.host.services.append(service)
+                service.host.services.append(weak_ref(service))
             
             sum1 += (time.time() - t52)
             
@@ -667,6 +691,11 @@ class Regenerator(object):
         t15 = time.time()
         logger_perf.info(u'clean old objects --------------------------------------------------------- : %.2f' % (t15 - t14))
         
+        hosts_and_services_to_refresh = [i for i in self.all_hosts_and_services if i.part_configuration_incarnation == part_configuration_incarnation]
+        StaticCache.refresh(hosts_and_services_to_refresh, part_configuration_incarnation.get_uuid())
+        t16 = time.time()
+        logger_perf.info(u'refresh static cache for %8i hosts and services ---------------------- : %.2f' % (len(hosts_and_services_to_refresh), t16 - t15))
+        
         nb_clusters = len([h for h in self.hosts if h.got_business_rule])
         self.item_counter = {
             u'hosts'           : len(self.hosts) - nb_clusters,
@@ -717,16 +746,53 @@ class Regenerator(object):
         
         start_time = time.time()
         
-        self.indexed_items = IndexedCollection()
-        for host in self.all_hosts_and_services:
-            self.indexed_items.append(host)
+        reindex_count = 0
+        reindex_time = 0
+        find_time = 0
+        for item in self.all_hosts_and_services:
+            uuid = item.get_instance_uuid()
+            
+            t0 = time.time()
+            item_in_indexed_items = self.indexed_items_2.get(uuid, None)
+            find_time += time.time() - t0
+            if self.has_change_for_item_in_indexed_items(item, item_in_indexed_items):
+                t0 = time.time()
+                if item_in_indexed_items:
+                    self.indexed_items.remove({u'instance_uuid': uuid})
+                self.indexed_items.append(item)
+                self.indexed_items_2[uuid] = item
+                reindex_count += 1
+                reindex_time += time.time() - t0
         
+        self.logger.info(u'generated indexed %s item list cache in %.3fs, with %s in this regenerator with %d total elements in indexed_items reindex_time:%.3fs find_time:%.3fs' % (
+            reindex_count, time.time() - start_time, len(self.all_hosts_and_services), self.indexed_items.counter, reindex_time, find_time))
+    
+    
+    def has_change_for_item_in_indexed_items(self, item, item_in_indexed_items):
+        if not item_in_indexed_items:
+            return True
+        if item_in_indexed_items is item:
+            return False  # Watch it it is False
+        
+        if item.get_name() != item_in_indexed_items.get_name():
+            return True
+        if item.get_realm() != item_in_indexed_items.get_realm():
+            return True
+        if self.get_type(item) != self.get_type(item_in_indexed_items):
+            return True
+        if self.get_contact_names(item) != self.get_contact_names(item_in_indexed_items):
+            return True
+        return False
+    
+    
+    @staticmethod
+    def get_type(item):
+        return u'cluster' if (item.get_item_type() == u'host' and item.got_business_rule) else item.get_item_type()
+    
+    
+    def get_contact_names(self, item):
         _def_access = self.get_default_property_value(u'host', u'view_contacts', missing=VIEW_CONTACTS_DEFAULT_VALUE.NOBODY)
-        self.indexed_items.add_index(u'contact_name', self._get_contact_list_default_everyone if _def_access == VIEW_CONTACTS_DEFAULT_VALUE.EVERYONE else self._get_contact_list, on_list=True)
-        self.indexed_items.add_index(u'type', lambda _h: u'cluster' if (_h.get_item_type() == u'host' and _h.got_business_rule) else _h.get_item_type())
-        self.indexed_items.add_index(u'realm', lambda _h: _h.get_realm())
-        
-        self.logger.info(u'generated indexed item list cache in %.3fs, with %d elements' % (time.time() - start_time, self.indexed_items.counter))
+        return self._get_contact_list_default_everyone(item) if _def_access == VIEW_CONTACTS_DEFAULT_VALUE.EVERYONE else self._get_contact_list(item)
     
     
     @staticmethod
@@ -739,7 +805,7 @@ class Regenerator(object):
     
     
     def indexed_items_lookup(self, lookup_filter, user):
-        # type: (Regenerator, Dict, Contact) -> List[Union[Host,Service]]
+        # type: (Regenerator, Dict, Contact) -> List[Union[BrokerHost,BrokerService]]
         if not self.with_indexed:
             raise NotImplementedError
         
@@ -757,7 +823,7 @@ class Regenerator(object):
             return
         cmdname = cc.command
         c = self.commands.find_by_name(cmdname)
-        cc.command = c
+        cc.command = weak_ref(c) if c else c
     
     
     # We look at o.prop and for each command we relink it
@@ -770,8 +836,9 @@ class Regenerator(object):
         
         for cc in v:
             cmdname = cc.command
-            c = self.commands.find_by_name(cmdname)
-            cc.command = c
+            if isinstance(cmdname, basestring):
+                c = self.commands.find_by_name(cmdname)
+                cc.command = weak_ref(c) if c else c
     
     
     # We look at the timeperiod() object of o.prop and we replace it with our true one
@@ -782,7 +849,7 @@ class Regenerator(object):
             return
         tpname = t.timeperiod_name
         tp = self.timeperiods.find_by_name(tpname)
-        setattr(o, prop, tp)
+        setattr(o, prop, weak_ref(tp) if tp else tp)
     
     
     # same than before, but the value is a string here
@@ -792,7 +859,7 @@ class Regenerator(object):
             setattr(o, prop, None)
             return
         tp = self.timeperiods.find_by_name(tpname)
-        setattr(o, prop, tp)
+        setattr(o, prop, weak_ref(tp) if tp else tp)
     
     
     # We look at o.prop and for each contacts in it,  we replace it with true object in self.contacts
@@ -806,7 +873,7 @@ class Regenerator(object):
         for cname in v:
             c = self.contacts.find_by_name(cname)
             if c:
-                new_v.append(c)
+                new_v.append(weak_ref(c))
         setattr(o, prop, new_v)
     
     
@@ -825,11 +892,13 @@ class Regenerator(object):
             sdesc = '/'.join(elts[1:])
             s = self.services.find_srv_by_name_and_hostname(hname, sdesc)
             if s:
+                if not isinstance(s, WeakRef):
+                    s = weak_ref(s)
                 new_v.append(s)
         for hname in v['hosts']:
             h = self.hosts.find_by_name(hname)
             if h:
-                new_v.append(h)
+                new_v.append(weak_ref(h))
         setattr(o, prop, new_v)
     
     
@@ -843,14 +912,14 @@ class Regenerator(object):
         for hname in v:
             h = self.hosts.find_by_name(hname)
             if h:
-                new_v.append(h)
+                new_v.append(weak_ref(h))
         setattr(o, prop, new_v)
     
     
     # ------- Brok management part
     
     def initial_status_freshness_check(self, item):
-        # type: (Regenerator, Union[Host, Service]) -> None
+        # type: (Regenerator, Union[BrokerHost, BrokerService]) -> None
         if not self.freshness_thread:
             return
         with self.freshness_lock:
@@ -994,7 +1063,7 @@ class Regenerator(object):
     def do_manage_program_status_brok(self, b):
         data = b.data
         instance_id = data['instance_id']
-        part_configuration_incarnation = b.part_configuration_incarnation
+        part_configuration_incarnation = b.part_configuration_incarnation  # type: PartConfigurationIncarnation
         logger_regenerator = self.logger.get_sub_part(u'scheduler=%s' % part_configuration_incarnation.scheduler_name, register=False)
         logger_regenerator.info(u'Creating new configuration for %s ' % part_configuration_incarnation)
         
@@ -1041,9 +1110,46 @@ class Regenerator(object):
         # We now regenerate reversed list so the client will find only real objects
         self.create_reversed_list()
         
+        # TODO: Warning this may have a severe cost, to be analyzed
+        self.all_hosts_and_services = []
+        self.all_hosts_and_services.extend(self.hosts)
+        self.all_hosts_and_services.extend(self.services)
+        self.all_hosts_and_services.sort(key=get_element_full_name)
+        
         logger_regenerator.info(u'clean %d hosts from previous configuration' % len(to_del_h))
         logger_regenerator.info(u'clean %d checks from previous configuration' % len(to_del_srv))
-        force_memory_trimming()
+        
+        # DEBUG pour trouver les références sur un hôte (voir plus bas)
+        # test_host = to_del_h[0] if to_del_h else None
+        
+        del to_del_h
+        del to_del_srv
+        clear_user_access_cache()
+        
+        # DEBUG pour trouver les références sur un objet
+        # if test_host:
+        #     import gc
+        #     try:
+        #         referrers = gc.get_referrers(test_host)
+        #         logger_regenerator.error(u'REF[%s] => %s' % (test_host.get_full_name(), referrers))
+        #         for item in referrers:
+        #             if isinstance(item, Host) or isinstance(item, Service):
+        #                 name = item.get_full_name()
+        #                 if not name:
+        #                     name = item.get_instance_uuid()
+        #                 if not name:
+        #                     name = '%s / %s' % (item, item.id)
+        #                 logger_regenerator.error(u'REF[%s] > %s' % (test_host.get_full_name(), name))
+        #             elif isinstance(item, list):
+        #                 logger_regenerator.error(u'REF[%s] > LIST:%s' % (test_host.get_full_name(), item))
+        #             else:
+        #                 logger_regenerator.error(u'REF[%s] > type:%s' % (test_host.get_full_name(), type(item)))
+        #
+        #     except:
+        #         logger_regenerator.print_stack()
+        #         pass
+        # else:
+        #     logger_regenerator.error(u'REF[None] > ')
     
     
     def manage_proxy_items_graph_brok(self, b):
@@ -1065,7 +1171,8 @@ class Regenerator(object):
             return
         
         # Whatever the data is giving us as id, create our own increasing one
-        h = Host({}, skip_useless_in_configuration=True)  # we do not care about running properties, they will be take from data
+        h = BrokerHost({}, skip_useless_in_configuration=True)  # we do not care about running properties, they will be take from data
+        h.list_cache_var = {}
         self.before_after_hook(b, h)
         self.update_element(h, data)
         self.initial_status_freshness_check(h)
@@ -1115,7 +1222,8 @@ class Regenerator(object):
             logger.print_stack()
             return
         
-        s = Service({}, skip_useless_in_configuration=True)  # we do not care about running properties, they will be take from data
+        s = BrokerService({}, skip_useless_in_configuration=True)  # we do not care about running properties, they will be take from data
+        s.list_cache_var = {}
         self.before_after_hook(b, s)
         self.update_element(s, data)
         self.initial_status_freshness_check(s)
@@ -1160,14 +1268,14 @@ class Regenerator(object):
     # We don't -> we create it
     # In both cases we need to relink it
     def manage_initial_contact_status_brok(self, brok):
-        if u'manage_initial_contact_status_brok' not in self._detail_timer:
-            self._detail_timer[u'manage_initial_contact_status_brok'] = {
+        if u'manage_initial_contact_status_brok' not in self.detail_timer:
+            self.detail_timer[u'manage_initial_contact_status_brok'] = {
                 u'build element'         : 0,
                 u'build notificationways': 0,
                 u'build contactgroups'   : 0,
                 u'build reversed list'   : 0,
             }
-        detail_timer = self._detail_timer[u'manage_initial_contact_status_brok']
+        detail_timer = self.detail_timer[u'manage_initial_contact_status_brok']
         
         contact_to_reindex = []
         notificationway_to_reindex = []
