@@ -25,23 +25,28 @@
 """ This python module contains the class BaseModule
 that shinken modules will subclass
 """
+
 import datetime
 import inspect
 import os
 import signal
+import threading
 import time
 from datetime import datetime
-from multiprocessing import Queue, Process
+from multiprocessing import Process
 
 from shinken.basesubprocess import BaseSubProcess, CommandQueueHandler, UnknownCommandException
+from shinken.ipc.shinken_queue.shinken_queue import build_shinken_queue, ShinkenQueue
 from shinken.log import logger as _logger, PART_INITIALISATION, LoggerFactory
-from shinken.misc.type_hint import Optional, TYPE_CHECKING
+from shinken.misc.type_hint import TYPE_CHECKING
 from shinken.modules.base_module.module_info_reader import ModuleInfoReaderMixin
 from shinken.objects.module import Module as ShinkenModuleDefinition
 from shinken.util import mem_wait_for_fork_possible
 
 if TYPE_CHECKING:
     from shinken.log import PartLogger
+    from shinken.inter_daemon_message import InterDaemonMessage
+    from shinken.misc.type_hint import List, NoReturn, Optional
 
 # TODO: use a class for defining the module "properties" instead of plain dict??  Like:
 '''
@@ -56,7 +61,7 @@ class ModuleProperties(object):
 properties = ModuleProperties('the_module_type', the_module_phases, is_mod_ext)
 '''
 
-# The properties dict defines what the module can do and if it's an external module or not.
+# The properties' dict defines what the module can do and if it's an external module or not.
 properties = {
     'type'    : None,  # name of the module type ; to distinguish between them:
     'external': True,  # is the module "external" (external means here a daemon module)?
@@ -72,10 +77,10 @@ class ModulePhases(object):
 
 
 class ModuleState(object):
-    OK = 'OK'
-    WARNING = 'WARNING'
-    CRITICAL = 'CRITICAL'
-    FATAL = 'FATAL'
+    OK = u'OK'
+    WARNING = u'WARNING'
+    CRITICAL = u'CRITICAL'
+    FATAL = u'FATAL'
 
 
 MAX_MODULE_INIT_TRY_INTERVAL = 60
@@ -121,7 +126,7 @@ class SOURCE_STATE(object):
         for s in SOURCE_STATE.STATES_ORDER:
             if s in (state1, state2):
                 return s
-        raise Exception('state [%s] [%s] not in STATES_ORDER' % (state1, state2))
+        raise Exception(u'state [%s] [%s] not in STATES_ORDER' % (state1, state2))
 
 
 class BaseModule(BaseSubProcess, ModuleInfoReaderMixin):
@@ -149,25 +154,17 @@ class BaseModule(BaseSubProcess, ModuleInfoReaderMixin):
         
         self.logger = LoggerFactory.get_logger(mod_conf.get_name())
         
-        # We can have sub modules
+        # We can have submodules
         self.modules = getattr(mod_conf, 'modules', [])
-        self.props = mod_conf.properties.copy()
         self.module_type = mod_conf.properties[u'type']
-        # TODO: choose between 'props' or 'properties'..
         self.interrupted = False
-        self.properties = self.props
-        self.is_external = self.props.get('external', False)
-        self._module_on_type = self.props.get('daemons', False)
-        self._module_type = self.props.get('type', False)
-        # though a module defined with no phase is quite useless .
-        self.phases = self.props.get('phases', [])
-        self.phases.append(None)
+        
         # the queue the module will receive data to manage
-        self.to_q = None  # type: Optional[Queue]
+        self.to_q = None  # type: Optional[ShinkenQueue]
         # the queue the module will put its result data
-        self.from_module_to_main_daemon_queue = None
-        self.commands_from_q = None
-        self.commands_to_q = None
+        self.from_module_to_main_daemon_queue = None  # type: Optional[ShinkenQueue]
+        self.commands_from_q = None  # type: Optional[ShinkenQueue]
+        self.commands_to_q = None  # type: Optional[ShinkenQueue]
         self.command_queue_handler = None  # type: Optional[CommandQueueHandler]
         self.process = None
         
@@ -186,9 +183,12 @@ class BaseModule(BaseSubProcess, ModuleInfoReaderMixin):
         self.manager = None
         
         # For sub process clean, we will have to keep a trace of all our queues
-        self._all_queues = []
+        self._all_queues = []  # type: List[Optional[ShinkenQueue]]
         
         self.fatal_error_has_been_managed = False
+        
+        self.messages_to_send_at_arbiter_lock = threading.RLock()
+        self.messages_to_send_at_arbiter = []  # type: List[InterDaemonMessage]
     
     
     def get_internal_state(self):
@@ -281,17 +281,9 @@ class BaseModule(BaseSubProcess, ModuleInfoReaderMixin):
         self.logger.info('The queue manager process is now clean.')
     
     
-    @staticmethod
-    def _get_new_queue(manager):
-        # If no Manager() object, go with classic Queue()
-        if not manager:
-            return Queue()
-        else:
-            return manager.Queue()
-    
-    
-    def queue_factory(self, name):
-        queue = self.manager.Queue()
+    def queue_factory(self, name, main_name=None, other_side_name=None):
+        # type: (unicode, Optional[unicode], Optional[unicode]) -> ShinkenQueue
+        queue = ShinkenQueue(name, main_name, other_side_name)
         queue.name = name
         self._all_queues.append(queue)
         self.logger.debug(PART_INITIALISATION, 'Creating a exchange queue [%s] with the module. Queue address (for debug only): %s ' % (name, queue))
@@ -299,74 +291,21 @@ class BaseModule(BaseSubProcess, ModuleInfoReaderMixin):
     
     
     def create_queues(self, manager_factory=None):
-        """The manager is None on android, but a true Manager() elsewhere
-        Create the shared queues that will be used by shinken daemon
-        process and this module process.
-        But clear queues if they were already set before recreating new one.
-        """
-        # We always kill the manager, to void it's queue and be sure we did release the memory
-        if self.manager:
-            self.kill_manager()
-        
-        if not self.manager:
-            self.logger.get_sub_part(PART_INITIALISATION).info(u'Creating a queue manager process for the module.')
-            self.manager = manager_factory('- Module: %s' % self.get_name())
         self.clear_queues()
-        
-        self.from_module_to_main_daemon_queue = self.queue_factory('from_module_to_main_daemon_queue')
-        self.to_q = self.queue_factory('to_q')
-        
-        self.commands_from_q = self.queue_factory('commands_from_q')
-        self.commands_to_q = self.queue_factory('commands_to_q')
+        self.logger.get_sub_part(PART_INITIALISATION).debug(u'Creating module[%s] queues.' % self.get_name())
+        module_name = self.get_name()
+        self.from_module_to_main_daemon_queue, self.to_q = build_shinken_queue(u'BRK', u'Main', u'%s' % module_name)
+        self.commands_from_q, self.commands_to_q = build_shinken_queue(u'CMD', u'Main', u'%s' % module_name)
         self.command_queue_handler = CommandQueueHandler(self.get_name(), self.logger, self.commands_to_q, self.commands_from_q, self)
+        self._all_queues.append(self.from_module_to_main_daemon_queue)
+        self._all_queues.append(self.commands_from_q)
     
     
     # Release the resources associated to the queues of this instance
-    # noinspection -> Because we HACK multiprocessing
-    # noinspection PyUnresolvedReferences
-    # noinspection PyProtectedMember
     def clear_queues(self):
-        from multiprocessing import util as multiprocessing_util
-        to_del = []
         for queue in self._all_queues:
-            if queue is None:
-                return
-            # HACK: the manager can be dead, and so the queue sockets will be unavailable, and will
-            #       40s timeout if we try to talk to then, so must remove them in the multiprocessing clean pass
-            # WARNING: multiprocessing_util._afterfork_registry can have parallel access, so loop until we have keys
-            registry_keys = None
-            watch_dog = time.time()
-            while registry_keys is None:
-                if time.time() - watch_dog > 10:
-                    registry_keys = []
-                    break
-                try:
-                    registry_keys = multiprocessing_util._afterfork_registry.keys()
-                except RuntimeError:  # oups, should retry
-                    registry_keys = []
-                    pass
-            for k in registry_keys:
-                obj = multiprocessing_util._afterfork_registry.get(k, None)
-                if obj is None:  # did disappears
-                    continue
-                if obj == queue:
-                    self.logger.debug('Disabling old [%s] queue. Queue address: %s (for debug only)' % (queue.name, queue))
-                    to_del.append(k)
-            
-            
-            def after_fork_skip(_queue):
-                pass
-            
-            
-            queue._incref = after_fork_skip
-            queue._token.address = None
-            queue._manager = None
-        for k in to_del:
-            try:
-                del multiprocessing_util._afterfork_registry[k]
-            except KeyError:  # already removed
-                pass
-        # And AFTER clean the afterfork_registry we can drop our pointer, and so maybe the registry as weakref will lose it
+            queue.close()
+        
         self.to_q = self.from_module_to_main_daemon_queue = self.commands_from_q = self.commands_to_q = None
         self._all_queues = []
     
@@ -389,18 +328,9 @@ class BaseModule(BaseSubProcess, ModuleInfoReaderMixin):
             raise Exception('[PERFORMANCE][MEMORY] Cannot start the module process as there is not enough memory for this')
         p = Process(target=self._main, args=())
         
-        # Under windows we should not call start() on an object that got
-        # its process as object, so we remove it and we set it in a earlier
-        # start
-        try:
-            del self.properties['process']
-        except:
-            pass
-        
         p.start()
         # We save the process data AFTER the fork()
         self.process = p
-        self.properties['process'] = p  # TODO: temporary
         self.logger.info(PART_INITIALISATION, "module process is started as pid=%d" % p.pid)
     
     
@@ -426,7 +356,7 @@ class BaseModule(BaseSubProcess, ModuleInfoReaderMixin):
     
     def is_manager_alive(self):
         if not self.manager:
-            return False
+            return True
         process = getattr(self.manager, '_process', None)
         if not process:
             return False
@@ -442,7 +372,7 @@ class BaseModule(BaseSubProcess, ModuleInfoReaderMixin):
         
         try:
             return self.process.is_alive() and self.is_manager_alive()
-        except AssertionError:  # maybe the process did go daemon and so we are no more its parent. Kill it any way
+        except AssertionError:  # maybe the process became daemon, so we are no more its parent. Kill it anyway
             self.logger.warning('Seems that the manager of the module is in a bad state')
             return False
     
@@ -455,29 +385,30 @@ class BaseModule(BaseSubProcess, ModuleInfoReaderMixin):
             if int(time.time()) > self._last_init_try + 3600:
                 self._last_init_try = int(time.time())
             if int(time.time()) < self._last_init_try + retry_interval:
-                _logger = module_manager_logger.get_sub_part(self.get_name()).get_sub_part(u'module-type=%s' % self._module_type, register=False)
-                _logger.debug(u'The module is not ready to start')
+                module_logger = module_manager_logger.get_sub_part(self.get_name()).get_sub_part(u'module-type=%s' % self._module_type, register=False)
+                module_logger.debug(u'The module is not ready to start')
                 return INSTANCE_INIT_STATE.INIT_SKIPPED
         return INSTANCE_INIT_STATE.INIT_READY
     
     
     def do_init(self, module_manager_logger):
-        _logger = module_manager_logger.get_sub_part(self.get_name(), register=False).get_sub_part(u'module-type=%s' % self._module_type, register=False)
+        module_logger = module_manager_logger.get_sub_part(self.get_name(), register=False).get_sub_part(u'module-type=%s' % self._module_type, register=False)
         self._second_interval_init_try += 10
         self._init_try_count += 1
         self._last_init_try = int(time.time())
         try:
-            _logger.info(u'Trying to init module.')
+            module_logger.info(u'Trying to init module.')
             self.init()
             self._second_interval_init_try = 20
             self._init_try_count = 1
         except Exception as exp:
-            _logger.error(u'The module failed to init and he raised an exception: %s.' % (str(exp)))
-            _logger.print_stack()
-            _logger.warning(u'The module tried to start: %s times.' % self._init_try_count)
-            _logger.warning(u'The module will try to restart at %s.' % (datetime.fromtimestamp(self._last_init_try + self._second_interval_init_try).strftime(u'%H:%M:%S')))
+            module_logger.error(u'The module failed to init and he raised an exception: %s.' % (str(exp)))
+            module_logger.print_stack()
+            if self.get_internal_state() != ModuleState.FATAL:
+                module_logger.warning(u'The module tried to start: %s times.' % self._init_try_count)
+                module_logger.warning(u'The module will try to restart at %s.' % (datetime.fromtimestamp(self._last_init_try + self._second_interval_init_try).strftime(u'%H:%M:%S')))
             return True, str(exp)
-        _logger.info(u'The module is started.')
+        module_logger.info(u'The module is started.')
         return False, u''
     
     
@@ -485,11 +416,11 @@ class BaseModule(BaseSubProcess, ModuleInfoReaderMixin):
         self.logger.info("Stopping module process pid=%s " % process.pid)
         try:
             process.terminate()
-        except OSError:  # Maybe the process was already gone, like kill by the system with a OOM or an angry admin
+        except OSError:  # Maybe the process was already gone, like kill by the system with an OOM or an angry admin
             pass
         try:
             process.join(timeout=1)
-        except OSError:  # Maybe the process was already gone too, but i prefer to be sure to try a join()
+        except OSError:  # Maybe the process was already gone too, but I prefer to be sure to try a join()
             pass
         if process.is_alive():
             self.logger.info("The sub-process [%s] is still alive, I help it to die" % process.pid)
@@ -604,6 +535,11 @@ class BaseModule(BaseSubProcess, ModuleInfoReaderMixin):
         raise NotImplementedError()
     
     
+    def loop_turn(self):
+        # type: () -> NoReturn
+        raise NotImplementedError()
+    
+    
     # Set by worker based modules to display their worker in the process list
     # noinspection -> real methode is not Static
     # noinspection PyMethodMayBeStatic
@@ -615,6 +551,10 @@ class BaseModule(BaseSubProcess, ModuleInfoReaderMixin):
         super(BaseModule, self)._start_sub_process_threads()
         
         self.command_queue_handler.start_thread()
+        
+        # Ensure that all our queues have their threads, so we don't have to pop them at runtime
+        for queue in self._all_queues:
+            _ = queue.get_queues_size()  # this will start the needed threads for this queue
     
     
     # module 'main' method. Only used by external modules.
@@ -677,3 +617,22 @@ class BaseModule(BaseSubProcess, ModuleInfoReaderMixin):
                 return raw_stats
             except Exception as e:
                 self.logger.error('Fail in get_raw_stats with error : [%s]' % getattr(e, 'message', str(e)))
+    
+    
+    def handle_push_messages_by_arbiter(self, push_message_by_arbiter):
+        # type: (InterDaemonMessage) -> None
+        pass
+    
+    
+    def send_messages_to_arbiter(self, message_to_arbiter):
+        # type: (InterDaemonMessage) -> None
+        with self.messages_to_send_at_arbiter_lock:
+            self.messages_to_send_at_arbiter.append(message_to_arbiter)
+    
+    
+    def get_messages_to_send_at_arbiter(self):
+        # type: () -> List[InterDaemonMessage]
+        with self.messages_to_send_at_arbiter_lock:
+            ret = self.messages_to_send_at_arbiter
+            self.messages_to_send_at_arbiter = []
+        return ret
