@@ -22,9 +22,9 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with Shinken.  If not, see <http://www.gnu.org/licenses/>.
 
-""" This is the main class for the Host. In fact it's mainly
+""" This is the main class for the Host. In fact, it's mainly
 about the configuration part. for the running one, it's better
-to look at the schedulingitem class that manage all
+to look at the SchedulingItem class that manage all
 scheduling/consume check smart things :)
 """
 
@@ -59,6 +59,7 @@ from shinkensolutions.service_override_parser import parse_service_override_prop
 if TYPE_CHECKING:
     from shinken.misc.type_hint import Optional
     from shinken.objects.command import Commands
+    from shinken.objects.hostgroup import Hostgroups
 
 SECONDS_MAX_DATE_VALUE = 157852800
 MINUTES_MAX_DATE_VALUE = 2630880
@@ -99,6 +100,60 @@ class Host(SchedulingItem):
     # the config.py "load_default_properties_values" method will fill it
     default_view_contacts_value = None
     
+    if TYPE_CHECKING:
+        max_check_attempts = 0
+        check_interval = 0
+        retry_interval = 0
+        obsess_over_host = False
+        low_flap_threshold = 25
+        high_flap_threshold = 50
+        flap_detection_enabled = True
+        host_name = u''
+        display_name = u''
+        realm = u''
+        # parents = None   # type: Optional[??]
+        hostgroups = None  # type: Optional[Hostgroups]
+        flap_detection_options = u''
+        notification_options = u''
+        # service_excludes = None   # type: Optional[??]
+        # service_excludes_by_id = None   # type: Optional[??]
+        service_overrides = u''
+
+        retain_status_information = True
+        retain_nonstatus_information = True
+        stalking_options = u''
+        notes = u''
+        action_url = u''
+        icon_image = u''
+        icon_image_alt = u''
+        icon_set = u''
+        vrml_image = u''
+        statusmap_image = u''
+        initial_state = u''
+        # 2d_coords
+        # 3d_coords
+        failure_prediction_enabled = False
+        labels = u''
+        business_rule_output_template = u''
+        business_rule_smart_notifications = False
+        business_rule_downtime_as_ack = False
+        business_rule_host_notification_options = u''
+        business_rule_service_notification_options = u''
+        time_to_orphanage = 300
+        in_inherited_downtime = False
+        childs = []
+        
+        
+        def __init__(self, params=None, skip_useless_in_configuration=False):
+            super(Host, self).__init__(params, skip_useless_in_configuration)
+            self.address = u''
+            self.alias = u''
+            self.last_time_unreachable = 0.0
+            self.last_time_up = 0.0
+            self.last_time_down = 0.0
+            self._list_of_service_uuids = []
+            self.templates = []
+    
     # properties defined by configuration
     # *required: is required in conf
     # *default: default value if no set in conf
@@ -107,44 +162,44 @@ class Host(SchedulingItem):
     # *no_slots: do not take this property for __slots__
     #  Only for the initial call
     # conf_send_preparation: if set, will pass the property to this function. It's used to "flatten"
-    #  some dangerous properties like realms that are too 'linked' to be send like that.
+    #  some dangerous properties like realms that are too 'linked' to be sent like that.
     # brok_transformation: if set, will call the function with the value of the property
     #  the major times it will be to flatten the data (like realm_name instead of the realm object).
     properties = SchedulingItem.properties.copy()
     properties.update({
         # Simple properties, can be set into a C structure
-        'max_check_attempts'    : IntegerProp(default='2', fill_brok=['full_status']),
-        'check_interval'        : IntegerProp(default='1', fill_brok=['full_status', 'check_result']),
-        'retry_interval'        : IntegerProp(default='1', fill_brok=['full_status', 'check_result']),
-        'obsess_over_host'      : BoolProp(default='0', fill_brok=['full_status']),
+        u'max_check_attempts'    : IntegerProp(default=u'2', fill_brok=[u'full_status']),
+        u'check_interval'        : IntegerProp(default=u'1', fill_brok=[u'full_status', u'check_result']),
+        u'retry_interval'        : IntegerProp(default=u'1', fill_brok=[u'full_status', u'check_result']),
+        u'obsess_over_host'      : BoolProp(default=u'0', fill_brok=[u'full_status']),
         
-        'low_flap_threshold'    : IntegerProp(default='25', fill_brok=['full_status']),
-        'high_flap_threshold'   : IntegerProp(default='50', fill_brok=['full_status']),
-        'flap_detection_enabled': BoolProp(default='1', fill_brok=['full_status']),
+        u'low_flap_threshold'    : IntegerProp(default=u'25', fill_brok=[u'full_status']),
+        u'high_flap_threshold'   : IntegerProp(default=u'50', fill_brok=[u'full_status']),
+        u'flap_detection_enabled': BoolProp(default=u'1', fill_brok=[u'full_status']),
         
         # Configuration Structure, like links to others elements
-        'host_name'             : StringProp(fill_brok=['full_status', 'check_result', 'next_schedule']),
-        'display_name'          : StringProp(default='', fill_brok=['full_status']),
-        'address'               : StringProp(fill_brok=['full_status'], inherit=False),
-        'realm'                 : StringProp(default=None, fill_brok=['full_status', 'check_result', 'next_schedule'], conf_send_preparation=get_obj_name),
-        'parents'               : LinkToOthersObjectsProp(brok_transformation=to_hostnames_list, default='', fill_brok=['full_status'], merging='join', handle_additive_inheritance=True),
-        'hostgroups'            : LinkToOthersObjectsProp(brok_transformation=to_list_string_of_names, default='', fill_brok=['full_status'], merging='join', handle_additive_inheritance=True),
-        'check_command'         : StringProp(default='check-host-alive', fill_brok=['full_status']),
+        u'host_name'             : StringProp(fill_brok=[u'full_status', u'check_result', u'next_schedule']),
+        u'display_name'          : StringProp(default='', fill_brok=[u'full_status']),
+        u'address'               : StringProp(fill_brok=[u'full_status'], inherit=False),
+        u'realm'                 : StringProp(default=None, fill_brok=[u'full_status', u'check_result', u'next_schedule'], conf_send_preparation=get_obj_name),
+        u'parents'               : LinkToOthersObjectsProp(brok_transformation=to_hostnames_list, default='', fill_brok=['full_status'], merging='join', handle_additive_inheritance=True),
+        u'hostgroups'            : LinkToOthersObjectsProp(brok_transformation=to_list_string_of_names, default='', fill_brok=['full_status'], merging='join', handle_additive_inheritance=True),
+        u'check_command'         : StringProp(default=u'check-host-alive', fill_brok=[u'full_status']),
         
-        # Theses one can be size bounded
-        'flap_detection_options': ListProp(default='o,d,u', fill_brok=['full_status'], merging='join'),
-        'notification_options'  : ListProp(default='d,u,r,f', fill_brok=['full_status'], merging='join'),
+        # These one can be size bounded
+        u'flap_detection_options': ListProp(default=u'o,d,u', fill_brok=[u'full_status'], merging=u'join'),
+        u'notification_options'  : ListProp(default=u'd,u,r,f', fill_brok=[u'full_status'], merging=u'join'),
         
-        'service_excludes'      : LinkToOthersObjectsProp(default='', merging='join', inherit=False),
-        'service_excludes_by_id': LinkToOthersObjectsProp(default='', merging='join', inherit=False),
-        'service_overrides'     : ListProp(default='', merging='duplicate', split_on_coma=False, separator="-=#=-"),
+        u'service_excludes'      : LinkToOthersObjectsProp(default=u'', merging=u'join', inherit=False),
+        u'service_excludes_by_id': LinkToOthersObjectsProp(default=u'', merging=u'join', inherit=False),
+        u'service_overrides'     : ListProp(default=u'', merging=u'duplicate', split_on_coma=False, separator="-=#=-"),
         
     })
     
     # properties keep but we don't show differences
     passthrough = SchedulingItem.passthrough.copy()
     passthrough.update({
-        ### Remove from the properties list.
+        # ## Remove from the properties list.
         'retain_status_information'                 : BoolProp(default='1', fill_brok=['full_status']),
         'retain_nonstatus_information'              : BoolProp(default='1', fill_brok=['full_status']),
         'stalking_options'                          : ListProp(default='', fill_brok=['full_status']),
@@ -175,7 +230,7 @@ class Host(SchedulingItem):
         # Enforces child nodes notification options
         'business_rule_host_notification_options'   : ListProp(default='', fill_brok=['full_status']),
         'business_rule_service_notification_options': ListProp(default='', fill_brok=['full_status']),
-        ### still exists in the properties list.
+        # ## still exists in the properties list.
         'time_to_orphanage'                         : IntegerProp(default='300', fill_brok=['full_status']),
         
     })
@@ -193,8 +248,8 @@ class Host(SchedulingItem):
     # retention: save/load this property from retention
     running_properties = SchedulingItem.running_properties.copy()
     running_properties.update({
-        ##################### Purely scheduler or arbiter internal
-        # no brok ,to much links
+        # #################### Purely scheduler or arbiter internal
+        # no brok, too many links
         'services'                             : RawProp(default=[]),
         'got_default_realm'                    : BoolProp(default=False),
         'last_cluster_state_id'                : IntegerProp(default=-1, retention=True, useless_in_configuration=True),
@@ -204,7 +259,7 @@ class Host(SchedulingItem):
         # Cluster sons, exported for raw data only
         'cluster_members'                      : RawProp(default=[], fill_raw_data=[TYPE_RAW_DATA_FULL], brok_transformation=to_cluster_members_uuid_list_and_do_cache_result),
         
-        #################### Need brok sending
+        # ################### Need brok sending
         'host_uuid'                            : StringProp(default='', fill_brok=['full_status', 'check_result', 'next_schedule'], brok_transformation=(lambda ref, value: ref.uuid)),
         'instance_uuid'                        : StringProp(default='', fill_brok=['full_status'], brok_transformation=get_brok_instance_uuid),
         'state_id'                             : IntegerProp(default=3, fill_brok=['full_status', 'check_result'], raw_data_transformation=get_real_state_id, fill_raw_data=[TYPE_RAW_DATA_CHECK_RESULT, TYPE_RAW_DATA_FULL], retention=True,
@@ -317,9 +372,10 @@ class Host(SchedulingItem):
     
     # Fill address with host_name if not already set
     def fill_predictive_missing_parameters(self):
-        if hasattr(self, 'host_name') and not hasattr(self, 'address'):
+        # type: () -> None
+        if hasattr(self, u'host_name') and not hasattr(self, u'address'):
             self.address = self.host_name
-        if hasattr(self, 'host_name') and not hasattr(self, 'alias'):
+        if hasattr(self, u'host_name') and not hasattr(self, u'alias'):
             self.alias = self.host_name
     
     
@@ -458,12 +514,12 @@ class Host(SchedulingItem):
         if not self.is_tpl():
             try:
                 return self.host_name
-            except AttributeError:  # outch, no hostname
+            except AttributeError:  # ouch, no hostname
                 return 'UNNAMEDHOST'
         else:
             try:
                 return self.name
-            except AttributeError:  # outch, no name for this template
+            except AttributeError:  # ouch, no name for this template
                 return 'UNNAMEDHOSTTEMPLATE'
     
     
@@ -472,7 +528,7 @@ class Host(SchedulingItem):
         for hg in self.hostgroups:
             # naglog_result('info', 'get_groupname : %s %s %s' % (hg.id, hg.alias, hg.get_name()))
             # groupname = "%s [%s]" % (hg.alias, hg.get_name())
-            groupname = "%s" % (hg.alias)
+            groupname = "%s" % hg.alias
         return groupname
     
     
@@ -516,7 +572,7 @@ class Host(SchedulingItem):
     
     # Say if we got the other in one of your dep list
     def is_linked_with_host(self, other):
-        for (h, status, type, timeperiod, inherits_parent) in self.act_depend_of:
+        for (h, status, _type, timeperiod, inherits_parent) in self.act_depend_of:
             if h == other:
                 return True
         return False
@@ -526,17 +582,17 @@ class Host(SchedulingItem):
     def del_host_act_dependency(self, other):
         to_del = []
         # First we remove in my list
-        for (h, status, type, timeperiod, inherits_parent) in self.act_depend_of:
+        for (h, status, _type, timeperiod, inherits_parent) in self.act_depend_of:
             if h == other:
-                to_del.append((h, status, type, timeperiod, inherits_parent))
+                to_del.append((h, status, _type, timeperiod, inherits_parent))
         for t in to_del:
             self.act_depend_of.remove(t)
         
         # And now in the father part
         to_del = []
-        for (h, status, type, timeperiod, inherits_parent) in other.act_depend_of_me:
+        for (h, status, _type, timeperiod, inherits_parent) in other.act_depend_of_me:
             if h == self:
-                to_del.append((h, status, type, timeperiod, inherits_parent))
+                to_del.append((h, status, _type, timeperiod, inherits_parent))
         for t in to_del:
             other.act_depend_of_me.remove(t)
         
@@ -547,26 +603,26 @@ class Host(SchedulingItem):
         self.parent_dependencies.remove(other)
     
     
-    # Add a dependency for action event handler, notification, etc)
-    # and add ourself in it's dep list
+    # Add a dependency for action event handler, notification, etc.)
+    # and add ourselves in its dep list
     def add_host_act_dependency(self, h, status, timeperiod, inherits_parent):
         # I add him in MY list
         self.act_depend_of.append((h, status, 'logic_dep', timeperiod, inherits_parent))
-        # And I add me in it's list
+        # And I add me in its list
         h.act_depend_of_me.append((self, status, 'logic_dep', timeperiod, inherits_parent))
         
         # And the parent/child dep lists too
         h.register_son_in_parent_child_dependencies(self)
     
     
-    # Register the dependency between 2 service for action (notification etc)
+    # Register the dependency between 2 service for action (notification etc.)
     # but based on a BUSINESS rule, so on fact:
-    # ERP depend on database, so we fill just database.act_depend_of_me
+    # ERP depends on database, so we fill just database.act_depend_of_me
     # because we will want ERP mails to go on! So call this
     # on the database service with the srv=ERP service
     def add_business_rule_act_dependency(self, h, status, timeperiod, inherits_parent):
-        # first I add the other the I depend on in MY list
-        # I only register so he know that I WILL be a impact
+        # first I add the other I depend on in MY list
+        # I only register, so he knows that I WILL be an impact
         self.act_depend_of_me.append((h, status, 'business_dep',
                                       timeperiod, inherits_parent))
         
@@ -578,7 +634,7 @@ class Host(SchedulingItem):
     def add_host_chk_dependency(self, h, status, timeperiod, inherits_parent):
         # I add him in MY list
         self.chk_depend_of.append((h, status, 'logic_dep', timeperiod, inherits_parent))
-        # And I add me in it's list
+        # And I add me in its list
         h.chk_depend_of_me.append((self, status, 'logic_dep', timeperiod, inherits_parent))
         
         # And we fill parent/childs dep for brok purpose
@@ -653,7 +709,7 @@ class Host(SchedulingItem):
         self.last_state_update = now
         
         # we should put in last_state the good last state:
-        # if not just change the state by an problem/impact
+        # if not just change the state by a problem/impact
         # we can take current state. But if it's the case, the
         # real old state is self.state_before_impact (it's the TRUE
         # state in fact)
@@ -752,7 +808,7 @@ class Host(SchedulingItem):
     
     
     # Raise a log entry with a Notification alert like
-    # HOST NOTIFICATION: superadmin;server;UP;notify-by-rss;no output
+    # HOST NOTIFICATION: super admin;server;UP;notify-by-rss;no output
     def raise_notification_log_entry(self, n):
         contact = n.contact
         command = n.command_call
@@ -768,8 +824,8 @@ class Host(SchedulingItem):
                              state, command.get_name(), self.output))
     
     
-    # Raise a log entry with a Eventhandler alert like
-    # HOST NOTIFICATION: superadmin;server;UP;notify-by-rss;no output
+    # Raise a log entry with an Eventhandler alert like
+    # HOST NOTIFICATION: super admin;server;UP;notify-by-rss;no output
     def raise_event_handler_log_entry(self, command):
         if self.__class__.log_event_handlers:
             naglog_result('critical', "HOST EVENT HANDLER: %s;%s;%s;%s;%s"
@@ -828,7 +884,7 @@ class Host(SchedulingItem):
     
     
     # Is stalking?
-    # Launch if check is waitconsume==first time
+    # Launch if check is wait consume==first time
     # and if c.status is in self.stalking_options
     def manage_stalking(self, c):
         need_stalk = False
@@ -847,7 +903,7 @@ class Host(SchedulingItem):
             logger.info("Stalking %s: %s" % (self.get_name(), self.output))
     
     
-    # An host is in downtime only by itself, don't care about it's services
+    # A host is in downtime only by itself, don't care about its services
     def is_in_downtime(self):
         return self.in_scheduled_downtime
     
@@ -864,7 +920,7 @@ class Host(SchedulingItem):
         return next((downtime.author for downtime in self.downtimes if downtime.is_in_effect), '')
     
     
-    # An host is in downtime only by itself, don't care about it's services
+    # A host is in downtime only by itself, don't care about its services
     def is_acknowledged(self):
         return self.problem_has_been_acknowledged
     
@@ -922,7 +978,7 @@ class Host(SchedulingItem):
         self.act_depend_of_me.append((child, ['d', 'u', 's', 'f'], 'network_dep', None, True))
     
     
-    # Give data for checks's macros
+    # Give data for checks' macros
     def get_data_for_checks(self):
         return [self]
     
@@ -932,7 +988,7 @@ class Host(SchedulingItem):
         return [self]
     
     
-    # Give data for notifications'n macros
+    # Give data for notifications' macros
     def get_data_for_notifications(self, contact, n):
         return [self, contact, n]
     
@@ -1019,7 +1075,7 @@ class Host(SchedulingItem):
         return False
     
     
-    # Get a oc*p command if item has obsess_over_*
+    # Get an oc*p command if item has obsess_over_*
     # command. It must be enabled locally and globally
     def get_obsessive_compulsive_processor_command(self):
         cls = self.__class__
@@ -1083,7 +1139,7 @@ class Host(SchedulingItem):
         
         if not (only_one_level and level >= 1):
             for template in self.templates:
-                # If we have a template loop, we should not lost time here
+                # If we have a template loop, we should not lose time here
                 if template in already_seen_templates:
                     continue
                 already_seen_templates.add(template)
@@ -1097,13 +1153,13 @@ class Host(SchedulingItem):
                     try:
                         current_value_parse = parse_service_override_property(current_value)
                     except SyntaxServiceOverrideError as e:
-                        # Service override must be validate before. This log shouldn't be call.
+                        # Service override must be validated before. This log shouldn't be called.
                         logger.warning(u'Fail to parse service overrides in %s with error : %s' % (self.get_name(), e))
                         continue
                     try:
                         tpl_value_parse = parse_service_override_property(tpl_value)
                     except SyntaxServiceOverrideError as e:
-                        # Service override must be validate before. This log shouldn't be call.
+                        # Service override must be validated before. This log shouldn't be called.
                         logger.warning(u'Fail to parse service overrides in %s with error : %s' % (self.get_name(), e))
                         continue
                     for check, so_value in tpl_value_parse.iteritems():
@@ -1231,8 +1287,9 @@ class Host(SchedulingItem):
     # NOTE: the synchronizer can give us the name in 2 ways:
     # UUID => it found the real check, so we are sure that we must take this one
     #           We check for service duplicate for each if the dfe_key match
-    # NAME => it only have its name, like in duplicate foreach, so we fallback to this
-    def parse_service_exclusion_expr(self, name_expr):
+    # NAME => it only has its name, like in duplicate foreach, so we fall back to this
+    @staticmethod
+    def parse_service_exclusion_expr(name_expr):
         # name_expr for link format is :  uuid!UUID!dfe_key!DEF_KEY or uuid!UUID
         name_expr_split = name_expr.split('!')
         uuid_to_search = name_expr_split[1]
@@ -1247,15 +1304,15 @@ class Host(SchedulingItem):
         check_definition_on_this_template = self.get_in_template_service_uuids()
         # logger.debug('EXCLUSION: %s/%s is from this template %s?' % (service.host_name, service.get_name(), self.get_name()))
         # We need to check 2 cases:
-        # * the check is a genuine template check, so have it's own uuid
+        # * the check is a genuine template check, so have its own uuid
         # * the check came from a duplicate foreach, so its uuid is a new, and the template know only about the original one
         # so take the definition uuid because the template only know about this one, not the final object
         if service.get_uuid_from_definition_object() not in check_definition_on_this_template:
             # logger.debug('EXCLUSION: %s/%s is from this template %s? NON (%s != %s)' % (service.host_name, service.get_name(), self.get_name(), service.get_uuid_from_definition_object(), check_definition_on_this_template))
-            return EXCLUDE_FROM_TEMPLATES_RESULTS.DONT_HAVE  # don't fucking care
+            return EXCLUDE_FROM_TEMPLATES_RESULTS.DONT_HAVE  # don't care
         
-        # V02.07.05 : DFE check can't be exclude on host by template because there are generated by the host
-        # V02.07.06 : If the template exclude by name the common part of the check, the check should be exclude (SEF-7341)
+        # V02.07.05 : DFE check can't be excluded on host by template because there are generated by the host
+        # V02.07.06 : If the template exclude by name the common part of the check, the check should be excluded (SEF-7341)
         if service.is_dfe():
             service_description_no_dfe_name = service.get_name_without_dfe()
             by_pattern_template_service_excludes = self._get_my_service_excludes()
@@ -1285,13 +1342,13 @@ class Host(SchedulingItem):
     def is_check_exclude(self, service):
         service_description = getattr(service, 'service_description', '__NO_SERVICE_DESCRIPTION__')
         
-        # We have name paterns, and uuids list
+        # We have patterns' name, and uuids list
         by_pattern_template_service_excludes = self._get_my_service_excludes()
         by_id_template_service_excludes = self._get_my_service_excludes_by_id()
         if not by_pattern_template_service_excludes and not by_id_template_service_excludes:
             return False
         
-        # Name patern
+        # Name pattern
         for template_service_exclude in by_pattern_template_service_excludes:
             if self._is_matching_name_regex_star_only(service_description, template_service_exclude):
                 return True
@@ -1307,7 +1364,8 @@ class Host(SchedulingItem):
         return False
     
     
-    def _is_matching_name_regex_star_only(self, name_service, exclude_by_name_label):
+    @staticmethod
+    def _is_matching_name_regex_star_only(name_service, exclude_by_name_label):
         exclude_by_name_label_escape = re.escape(exclude_by_name_label)
         exclude_by_name_label_escape = exclude_by_name_label_escape.replace('\\*', ".*")
         return re.match(r'^%s$' % exclude_by_name_label_escape, name_service)
@@ -1344,7 +1402,7 @@ class Hosts(InheritableItems):
     
     
     # Create link between elements:
-    # hosts -> timeperiods
+    # hosts -> time periods
     # hosts -> hosts (parents, etc)
     # hosts -> commands (check_command)
     # hosts -> contacts
@@ -1361,8 +1419,8 @@ class Hosts(InheritableItems):
         self.linkify_with_contacts(contacts)
         self.linkify_with_resultmodulations(resultmodulations)
         self.linkify_with_business_impact_modulations(businessimpactmodulations)
-        # WARNING: all escalations will not be link here
-        # (just the escalation here, not serviceesca or hostesca).
+        # WARNING: all escalations will not be linked here
+        # (just the escalation here, not service esca or host esca).
         # This last one will be link in escalations linkify.
         self.linkify_with_escalations(escalations)
         # self.linkify_with_triggers(triggers)
@@ -1376,7 +1434,7 @@ class Hosts(InheritableItems):
             h.fill_predictive_missing_parameters()
     
     
-    # Link host with hosts (parents)
+    # Links host with hosts (parents)
     def linkify_h_by_h(self):
         for h in self:
             parents = h.parents
@@ -1447,7 +1505,8 @@ class Hosts(InheritableItems):
                 h.hostgroups = []
     
     
-    # We look for hostgroups property in hosts and
+    # We look for host groups property in hosts and
+    # noinspection PyUnusedLocal, SpellCheckingInspection
     def explode(self, hostgroups, contactgroups):
         # Register host in the hostgroups
         for h in self:
@@ -1536,7 +1595,7 @@ class Hosts(InheritableItems):
         # First find the template
         tpl = self.find_template(tpl_name)
         
-        # If we find none, we should manually lookup all hosts to find this 'tag'
+        # If we find none, we should manually look up all hosts to find this 'tag'
         if tpl is None:
             # NOP => DISABLING TAG research because it's too difficult for users to understand
             self.__template_to_host_cache[tpl_name] = []  # update the cache
@@ -1566,7 +1625,7 @@ class Hosts(InheritableItems):
                     continue
         
         res = list(res)  # need a list, not a set()
-        # update the cache so we don't do this research again
+        # update the cache, so we don't do this research again
         self.__template_to_host_cache[tpl_name] = res
         return res
     
@@ -1614,6 +1673,46 @@ class Hosts(InheritableItems):
         logger.info('[performance] - (detail for support only) explode::hosts::explode_contact_groups_into_contacts   %.2f' % (time.time() - t0))
     
     
+    def create_reversed_list(self):
+        # type: () -> None
+        InheritableItems.create_reversed_list(self)
+        
+        name_property = self.__class__.name_property
+        self.reversed_list_name_lower = {}
+        for _id, item in self.items.iteritems():
+            if hasattr(item, name_property):
+                name = getattr(item, name_property)
+                self.reversed_list_name_lower[name.lower()] = _id
+    
+    
+    def find_id_by_name_case_insensitive(self, name):
+        # type: (unicode) -> Optional[int]
+        
+        name_lower = name.lower()
+        if hasattr(self, 'reversed_list_name_lower'):
+            if name_lower in self.reversed_list_name_lower:
+                return self.reversed_list_name_lower[name_lower]
+            else:
+                return None
+        else:  # ok, an early ask, with no reversed list from now...
+            name_property = self.__class__.name_property
+            for i in self:
+                if hasattr(i, name_property):
+                    i_name = getattr(i, name_property)
+                    if i_name.lower() == name_lower:
+                        return i.id
+            return None
+    
+    
+    def get_host_case_insensitive(self, name):
+        # type: (unicode) -> Optional[Host]
+        _id = self.find_id_by_name_case_insensitive(name)
+        if _id is not None:
+            return self.items[_id]
+        else:
+            return None
+    
+    
     @staticmethod
     def _apply_inheritance_from_command(host, commands):
         # type: (Host, Commands) -> None
