@@ -22,14 +22,15 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with Shinken.  If not, see <http://www.gnu.org/licenses/>.
 
-""" This Class is the service one, s it manage all service specific thing.
+""" This Class is the service one, it manages all service specific thing.
 If you look at the scheduling part, look at the scheduling item class"""
 import hashlib
 import re
-import time
 from collections import deque
 from collections import namedtuple
 
+import time
+
 from shinken.autoslots import AutoSlots
 from shinken.eventhandler import EventHandler
 from shinken.log import logger, naglog_result
@@ -60,7 +61,8 @@ from shinken.util import (
 if TYPE_CHECKING:
     from shinken.objects.host import Host, Hosts
     from shinken.objects.command import Commands
-    from shinken.misc.type_hint import List
+    from shinken.objects.servicegroup import Servicegroups
+    from shinken.misc.type_hint import List, Optional
 
 SECONDS_MAX_DATE_VALUE = 157852800
 MINUTES_MAX_DATE_VALUE = 2630880
@@ -116,6 +118,48 @@ class Service(SchedulingItem):
     # name of the type when display log for end users
     my_type_log_display = 'check'
     
+    if TYPE_CHECKING:
+        is_volatile = False
+        max_check_attempts = 1
+        check_interval = 0
+        retry_interval = 0
+        obsess_over_service = False
+        low_flap_threshold = -1
+        high_flap_threshold = -1
+        flap_detection_enabled = True
+        merge_host_contacts = False
+        host_dependency_enabled = True
+        host_name = u''
+        hostgroup_name = u''
+        service_description = u''
+        display_name = u''
+        servicegroups = None  # type: Optional[Servicegroups]
+        check_command = u''
+        flap_detection_options = u''
+        notification_options = u''
+        service_dependencies = u''
+        duplicate_foreach = u''
+        default_value = u''
+        from_template = u''
+        
+        host = None  # type: Optional[Host]
+        state_before_hard_unknown_reach_phase = u''
+        last_cluster_state_id = -1
+        host_uuid = u''
+        instance_uuid = u''
+        state_id = 3
+        last_time_ok = 0
+        last_time_warning = 0
+        last_time_critical = 0
+        last_time_unknown = 0
+        problem_has_been_acknowledged = False
+        in_inherited_acknowledged = False
+        customs = {}
+        in_partial_downtime = False
+        in_scheduled_downtime = False
+        in_inherited_downtime = False
+        realm = u''
+    
     # properties defined by configuration
     # required: is required in conf
     # default: default value if no set in conf
@@ -149,7 +193,7 @@ class Service(SchedulingItem):
         
         'check_command'          : StringProp(fill_brok=['full_status']),
         
-        # Theses one can be size bound
+        # These one can be size bound
         'flap_detection_options' : ListProp(default='o,w,c,u', fill_brok=['full_status']),
         'notification_options'   : ListProp(default='d,u,r,f', fill_brok=['full_status'], merging='join'),  # same default than host
         
@@ -168,7 +212,7 @@ class Service(SchedulingItem):
     # properties keep but we don't show differences
     passthrough = SchedulingItem.passthrough.copy()
     passthrough.update({
-        ### Remove from the properties list.
+        # ## Remove from the properties list.
         'initial_state'                             : CharProp(default='o', fill_brok=['full_status']),
         
         'retain_status_information'                 : BoolProp(default='1', fill_brok=['full_status']),
@@ -197,7 +241,7 @@ class Service(SchedulingItem):
         
         'apply_on_type'                             : StringProp(default='', fill_brok=['full_status']),
         
-        ### still exists in the properties list.
+        # ## still exists in the properties list.
         'display_name'                              : StringProp(default='', fill_brok=['full_status']),
         'servicegroups'                             : StringProp(default='', fill_brok=['full_status'], brok_transformation=to_list_string_of_names, merging='join'),
         
@@ -218,12 +262,12 @@ class Service(SchedulingItem):
     # properties used in the running state
     running_properties = SchedulingItem.running_properties.copy()
     running_properties.update({
-        ##################### Purely scheduler or arbiter internal
+        # #################### Purely scheduler or arbiter internal
         'host'                                 : StringProp(default=None),
         'state_before_hard_unknown_reach_phase': StringProp(default='OK', retention=True),  # Manage the unknown/unreach during hard state
         'last_cluster_state_id'                : IntegerProp(default=-1, retention=True, useless_in_configuration=True),
         
-        #################### Need brok sending
+        # ################### Need brok sending
         'host_uuid'                            : StringProp(default='', fill_brok=['full_status', 'check_result', 'next_schedule']),
         'instance_uuid'                        : StringProp(default='', fill_brok=['full_status'], brok_transformation=get_brok_instance_uuid, fill_raw_data=[TYPE_RAW_DATA_CHECK_RESULT, TYPE_RAW_DATA_FULL]),
         
@@ -249,7 +293,7 @@ class Service(SchedulingItem):
         u'dfe_value_resolved'                  : RawProp(default={}),  # Present in Arbiter configuration to Scheduler
     })
     
-    # Mapping between Macros and properties (can be prop or a function)
+    # Mapping between Macros and properties (can be a property or a function)
     macros = {
         'SERVICEUUID'                  : 'get_instance_uuid',
         'SERVICEDESC'                  : 'service_description',
@@ -307,15 +351,24 @@ class Service(SchedulingItem):
         'criticity'            : 'business_impact',
         'hostgroup'            : 'hostgroup_name',
         'hostgroups'           : 'hostgroup_name',
-        ## 'criticitymodulations':    'business_impact_modulations',
+        # 'criticitymodulations':    'business_impact_modulations',
     }
     
     
-    def __init__(self, params={}, skip_useless_in_configuration=False):
+    def __init__(self, params=None, skip_useless_in_configuration=False):
+        if params is None:
+            params = {}
         super(Service, self).__init__(params, skip_useless_in_configuration)
+        
         # Private key use only in arbiter
         self._origin_uuid = u''  # uuid of the service use to make the duplicate for each
         self._dfe_original_name_without_key = u''
+        
+        if TYPE_CHECKING:
+            self.last_time_ok = 0.0
+            self.last_time_warning = 0.0
+            self.last_time_critical = 0.0
+            self.last_time_unknown = 0.0
     
     
     #######
@@ -526,7 +579,7 @@ class Service(SchedulingItem):
     # Must be AFTER linkify
     # TODO: implement "not host dependent" feature.
     def fill_daddy_dependency(self):
-        #  Depend of host, all status, is a networkdep
+        #  Depend on host, all status, is a network dep
         # and do not have timeperiod, and follow parents dep
         if self.host is not None and self.host_dependency_enabled:
             # I add the dep in MY list
@@ -550,7 +603,7 @@ class Service(SchedulingItem):
     
     # Register the dependency between 2 service for action (notification etc)
     def add_service_act_dependency(self, srv, status, timeperiod, inherits_parent):
-        # first I add the other the I depend on in MY list
+        # first I add the other I depend on in MY list
         self.act_depend_of.append((srv, status, 'logic_dep',
                                    timeperiod, inherits_parent))
         # then I register myself in the other service dep list
@@ -561,13 +614,13 @@ class Service(SchedulingItem):
         srv.register_son_in_parent_child_dependencies(self)
     
     
-    # Register the dependency between 2 service for action (notification etc)
+    # Register the dependency between 2 service for action (notification etc.)
     # but based on a BUSINESS rule, so on fact:
-    # ERP depend on database, so we fill just database.act_depend_of_me
+    # ERP depends on database, so we fill just database.act_depend_of_me
     # because we will want ERP mails to go on! So call this
     # on the database service with the srv=ERP service
     def add_business_rule_act_dependency(self, srv, status, timeperiod, inherits_parent):
-        # I only register so he know that I WILL be a impact
+        # I only register, so he knows that I WILL be an impact
         self.act_depend_of_me.append((srv, status, 'business_dep',
                                       timeperiod, inherits_parent))
         
@@ -577,7 +630,7 @@ class Service(SchedulingItem):
     
     # Register the dependency between 2 service for checks
     def add_service_chk_dependency(self, srv, status, timeperiod, inherits_parent):
-        # first I add the other the I depend on in MY list
+        # first I add the other I depend on in MY list
         self.chk_depend_of.append((srv, status, 'logic_dep',
                                    timeperiod, inherits_parent))
         # then I register myself in the other service dep list
@@ -644,7 +697,7 @@ class Service(SchedulingItem):
                         for (custom_key, custom_value) in new_s.customs.iteritems():
                             if dollar_key_string in custom_value:
                                 new_s.customs[custom_key] = custom_value.replace(dollar_key_string, key_value[_dfe_data_name])
-                    # s got the same uuid than the source service, it's a problem, so set it's uuid as
+                    # s got the same uuid as the source service, it's a problem, so set it's uuid as
                     # a hex about it's service description, the only part quite stable
                     if hasattr(new_s, u'service_description'):
                         new_s.uuid = hashlib.md5(new_s.service_description.encode(u'utf-8')).hexdigest()
@@ -674,7 +727,7 @@ class Service(SchedulingItem):
     #                                 |___/
     ####
     
-    # Set unreachable: our host is DOWN, but it mean nothing for a service
+    # Set unreachable: our host is DOWN, but it means nothing for a service
     def set_unreachable(self):
         pass
     
@@ -691,6 +744,7 @@ class Service(SchedulingItem):
             self.state_changed_since_impact = False
             self.state = 'UNKNOWN'  # exit code UNDETERMINED
             self.state_id = 3
+            self.last_chk = int(time.time())
             self.output = prepend_explaination_about_set_impact_to_check_output(self.output)
     
     
@@ -704,6 +758,7 @@ class Service(SchedulingItem):
             self.state = self.state_before_impact
             self.state_id = self.state_id_before_impact
             self.output = self.output_before_impact
+            self.last_chk = int(time.time())
             proxyitemsmgr.update_state(self.get_instance_uuid(), self.state_id)
         # In all cases, clean the output that can be big
         self.output_before_impact = ''
@@ -715,11 +770,11 @@ class Service(SchedulingItem):
         self.last_state_update = now
         
         # we should put in last_state the good last state:
-        # if not just change the state by an problem/impact
+        # if not just change the state by a problem/impact
         # we can take current state. But if it's the case, the
         # real old state is self.state_before_impact (it's the TRUE
         # state in fact)
-        # but only if the global conf have enable the impact state change
+        # but only if the global conf have enabled the impact state change
         cls = self.__class__
         if cls.enable_problem_impacts_states_change \
                 and self.is_impact \
@@ -804,7 +859,7 @@ class Service(SchedulingItem):
     # SERVICE ALERT: server;Load;UNKNOWN;HARD;1;I don't know what to say...
     def raise_alert_log_entry(self):
         # if self.attempt == 0:
-        #     raise Exception('Attempts cannot be 0 here! %s %s  state=%s  last_state=%s  state_type=%s  max_check_attempts=%s' % (self, self.get_dbg_name(), self.state, self.last_state, self.state_type, self.max_check_attempts))
+        #     raise Exception('Attempts cannot be 0 here! %s (%s)  state=%s  last_state=%s  state_type=%s  max_check_attempts=%s' % (self, self.get_dbg_name(), self.state, self.last_state, self.state_type, self.max_check_attempts))
         naglog_result('critical', 'SERVICE ALERT: %s;%s;%s;%s;%d;%s'
                       % (self.host.get_name(), self.get_name(),
                          self.state, self.state_type,
@@ -851,7 +906,7 @@ class Service(SchedulingItem):
                              command.get_name(), self.output))
     
     
-    # Raise a log entry with a Eventhandler alert like
+    # Raise a log entry with an Eventhandler alert like
     # SERVICE EVENT HANDLER: test_host_0;test_ok_0;OK;SOFT;4;eventhandler
     def raise_event_handler_log_entry(self, command):
         if self.__class__.log_event_handlers:
@@ -1095,7 +1150,7 @@ class Service(SchedulingItem):
         return False
     
     
-    # Get a oc*p command if item has obsess_over_*
+    # Get an oc*p command if item has obsess_over_*
     # command. It must be enabled locally and globally
     def get_obsessive_compulsive_processor_command(self):
         cls = self.__class__
@@ -1152,6 +1207,11 @@ class Services(InheritableItems):
         super(Services, self).__init__(items)
         self.__host_templates_expression_to_hosts_cache = {}
         self.__host_templates_expression_to_templates_cache = {}
+        
+        if TYPE_CHECKING:
+            self.reversed_list = {}
+            self.twins = []
+            self.hosts = None  # type: Optional[Hosts]
     
     
     def load_value_from_global_conf(self):
@@ -1160,12 +1220,11 @@ class Services(InheritableItems):
             service.load_value_from_global_conf()
     
     
-    # Create the reversed list for speedup search by host_name/name
-    # We also tag service already in list: they are twins. It's a a bad things.
-    # Hostgroups service have an ID higher than host service. So it we tag
-    # an id that already are in the list, this service is already
-    # exist, and is a host,
-    # or a previous hostgroup, but define before.
+    # Create the reversed list speeding up search by host_name/name
+    # We also tag service already in list: they are twins. It's a bad things.
+    # Host groups service have an ID higher than host service. So if we tag
+    # an id that already are in the list, this service already
+    # exists, and is a host, a previous host group, but defined before.
     def create_reversed_list(self):
         self.reversed_list = {}
         self.twins = []
@@ -1213,20 +1272,20 @@ class Services(InheritableItems):
             else:
                 reversed_list[key] = [self._get_service_priority(twin_id, host)]
         
-        # the we sort the list each key with the first and keep only the first if the priority is not the same as the second (or more)
+        # we sort the list each key with the first and keep only the first if the priority is not the same as the second (or more)
         for key, twins_services in reversed_list.iteritems():
             twins_services = sorted(twins_services, key=lambda k: k[0])
             if twins_services[0].priority == twins_services[1].priority:
                 self.configuration_errors.append("Impossible to guess which of the %d check named '%s' to apply to host '%s'." % (len(twins_services), key[1], key[0]))
             for _, twin_id in twins_services[1:]:
-                # here we delete all twin, EVEN the errors otherwise the twin list will be recompute and the logs will be ugly
+                # here we delete all twin, EVEN the errors otherwise the twin list will be recomputed and the logs will be ugly
                 del self[twin_id]
         
         self.twins = []
     
     
     def _get_service_priority(self, twin_id, host):
-        # We set the template level to max +1 because if the service not match any template, it must be attach to hostgroup
+        # We set the template level to max +1 because if the service not match any template, it must be attached to host group
         templates = []
         flatten_tpls(host, templates)
         template_level = len(templates) + 1
@@ -1235,7 +1294,7 @@ class Services(InheritableItems):
                 template_level = i
                 break
         
-        # The we take in account if a service is a DFE
+        # We take in account if a service is a DFE
         is_dfe = 1 if self[twin_id].is_dfe() else 0
         
         # Finally we set the priority with :
@@ -1290,14 +1349,14 @@ class Services(InheritableItems):
                 return None
             return h.find_service_by_name(name)
         
-        id = self.find_srv_id_by_name_and_hostname(host_name, name)
-        if id is not None:
-            return self.items[id]
+        _id = self.find_srv_id_by_name_and_hostname(host_name, name)
+        if _id is not None:
+            return self.items[_id]
         else:
             return None
     
     
-    # Search a service by it's uuid and host_uuid
+    # Search a service by its uuid and host_uuid
     def find_srv_by_host_uuid_and_check_uuid(self, host_uuid, check_uuid):
         if hasattr(self, 'hosts'):
             h = self.hosts.find_by_uuid(host_uuid)
@@ -1337,7 +1396,7 @@ class Services(InheritableItems):
                     continue
                 
                 # step 2 - Exclude service excluded by the host template
-                # If all templates which have the service excluded the service it will be remove
+                # If all templates which have the service excluded the service it will be removed
                 # So if one template do not excluded the service it will be keep
                 remove_by_template = False
                 for host_template in host.templates:
@@ -1346,7 +1405,7 @@ class Services(InheritableItems):
                     # if the template do not have it, we don't stop here
                     if template_result == EXCLUDE_FROM_TEMPLATES_RESULTS.DONT_HAVE:
                         continue
-                    # so now have it, must have a exclude or not
+                    # so now have it, must have an exclusion or not
                     # if it excludes, mark it, but all templates that have it must exclude it
                     elif template_result == EXCLUDE_FROM_TEMPLATES_RESULTS.EXCLUDE:
                         remove_by_template = True
@@ -1398,7 +1457,8 @@ class Services(InheritableItems):
         self.linkify_with_macromodulations(macromodulations)
     
     
-    def _search_service_for_host_link(self, host, name_expr, cache, cache_index):
+    @staticmethod
+    def _search_service_for_host_link(host, name_expr, cache, cache_index):
         # To speed up search if several properties are overridden on the same service, we keep them in temporary cache
         host_name = getattr(host, 'host_name', getattr(host, 'name'))
         key = "%s/%s" % (host_name, name_expr)
@@ -1418,7 +1478,7 @@ class Services(InheritableItems):
             # NOTE: the synchronizer can give us the name in 2 ways:
             # UUID => it found the real check, so we are sure that we must take this one
             #           We check for service duplicate for each if the dfe_key match
-            # NAME => it only have its name, like in duplicate foreach, so we fallback to this
+            # NAME => it only has its name, like in duplicate foreach, so we fall back to this
             if name_expr.startswith(PREFIX_LINK_UUID):
                 uuid_to_search, dfe_key = host.parse_service_exclusion_expr(name_expr)
                 link_match_service_without_dfe = (not s.dfe_key_resolved and s.uuid == uuid_to_search)
@@ -1483,7 +1543,7 @@ class Services(InheritableItems):
                 
                 service = self._search_service_for_host_link(host, name_expr, cache, _host_to_service_temporary_index)
                 
-                # If the service is missing, just means that the synchronizer did send us a useless information about
+                # If the service is missing, just means that the synchronizer did send us useless information about
                 # a disabled check or such thing. not a problem, UI will display the warning
                 if service is None:
                     logger.debug('NOTE: we cannot find the service for the override %s for the host %s' % (ovr, host.get_name()))
@@ -1527,7 +1587,7 @@ class Services(InheritableItems):
                                 delattr(service, prop)
                                 service.fill_default_one_property(prop)
                         else:
-                            # The value cannot be take from host, and what ever the service value was,
+                            # The value cannot be taken from host, and what ever the service value was,
                             # we want the default value one (we delete it and set the default value)
                             delattr(service, prop)
                             service.fill_default_one_property(prop)
@@ -1535,7 +1595,7 @@ class Services(InheritableItems):
                         setattr(service, prop, value)
     
     
-    # We can link services with hosts so
+    # We can link services with hosts, so
     # We can search in O(hosts) instead
     # of O(services) for common cases
     def optimize_service_search(self, hosts, build_service_on_host_index=True):
@@ -1736,8 +1796,8 @@ class Services(InheritableItems):
     
     
     # From a complex template expression, we grok templates that match it, so we know in the next step that
-    # the checks are from this templates, and if we must keep them from a exclude point of view (exclude is
-    # computed BY TEMPLATE so we must know which template came with a specific check)
+    # the checks are from these templates, and if we must keep them from an exclusion point of view (exclude is
+    # computed BY TEMPLATE, so we must know which template came with a specific check)
     def _tag_templates_with_service_definition_from_hosttemplates_expression(self, htemplates_expression, hosts, service_uuid):
         host_templates_names = self._get_templates_from_hosttemplates_expression(htemplates_expression, hosts)
         for host_templates_name in host_templates_names:
@@ -1801,16 +1861,16 @@ class Services(InheritableItems):
         # Then for every host create a copy of the service with just the host
         # because we are adding services, we can't just loop in it
         service_to_check = self.items.keys()
-        for id in service_to_check:
-            service = self.items[id]
+        for _id in service_to_check:
+            service = self.items[_id]
             duplicate_for_hosts = []  # get the list of our host_names if more than 1
-            not_hosts = []  # the list of !host_name so we remove them after
+            not_hosts = []  # the list of !host_name, so we remove them after
             
             # Services must have a name or service description
             if not getattr(service, 'service_description', getattr(service, 'name', '')):
                 self.configuration_errors.append("A service is missing description or name. Cannot apply configuration.")
             
-            # If do not have an host_name, just delete it
+            # If we do not have a host_name, just delete it
             if not getattr(service, 'host_name', ''):
                 srv_to_remove.append(service.id)
                 continue
@@ -1845,11 +1905,11 @@ class Services(InheritableItems):
                     if not service.is_tpl():
                         hosts_set.add(hname)
                     else:
-                        # For template it's more tricky: it's a template name
-                        # we've got, not a real host_name/ So we must get a list of host_name
+                        # For template, it's more tricky: it's a template name
+                        # we have, not a real host_name/ So we must get a list of host_name
                         # that use this template
                         # Use the complex expression manager for it, it will call find_hosts_that_use_template
-                        # for the templates it think it's useful
+                        # for the templates it thinks it's useful
                         htemplates_expression = hname
                         hosts_for_this_expression = self._get_hosts_from_hosttemplates_expression(htemplates_expression, hosts)
                         hosts_set = hosts_set.union(hosts_for_this_expression)
@@ -1860,7 +1920,7 @@ class Services(InheritableItems):
                 # Since we exploded the service, original service must be deleted.
                 # Keep ones that have configuration errors for error handling
                 if not service.configuration_errors:
-                    srv_to_remove.append(id)
+                    srv_to_remove.append(_id)
             
             else:  # Maybe the hnames was full of same host, so we must reset the name
                 for hname in hnames:  # So even if len == 0, we are protected
@@ -1873,7 +1933,7 @@ class Services(InheritableItems):
         logger.info('[performance] - (detail for support only) explode::services::delete_services_by_id   %.2f' % (time.time() - t0))
         
         t0 = time.time()
-        # Servicegroups property need to be fullfill for got the informations
+        # Service groups property need to be fulfilled to have the information
         # And then just register to this service_group
         for service in self:
             if not service.is_tpl() and hasattr(service, 'service_description'):
