@@ -39,9 +39,10 @@ import psutil
 
 from shinken.log import logger
 from shinken.misc.type_hint import TYPE_CHECKING
+from shinkensolutions.date_helper import get_now
 
 if TYPE_CHECKING:
-    from shinken.misc.type_hint import Optional
+    from shinken.misc.type_hint import Any, Dict, List, Optional, Tuple, Union
 
 try:
     from ClusterShell.NodeSet import NodeSet, NodeSetParseRangeError
@@ -62,6 +63,56 @@ except Exception as exp:
 PAGESIZE = 0
 
 
+# NOTE: This object is NOT meant to be a singleton (because of possible fork issues)
+# It must be a local object attached to a module, a daemon, or whatever.
+class DelayedForceMemoryTrimManagerByPeriod(object):
+    __slots__ = ('_last_force_trim_time', '_minimum_time_before_next_force_trim')
+    
+    
+    def __init__(self, minimum_time_before_next_force_trim):
+        # type: (float) -> None
+        self._last_force_trim_time = 0  # type: float
+        self._minimum_time_before_next_force_trim = minimum_time_before_next_force_trim  # type: float
+    
+    
+    def ask_for_memory_trim(self, context):
+        # type: (unicode) -> None
+        now = get_now()
+        if now - self._last_force_trim_time < self._minimum_time_before_next_force_trim:
+            return
+        self._last_force_trim_time = now
+        force_memory_trimming(context)
+
+
+# NOTE: This object is NOT meant to be a singleton (because of possible fork issues)
+# It must be a local object attached to a module, a daemon, or whatever.
+class DelayedForceMemoryTrimManagerByCounter(object):
+    __slots__ = ('_last_force_trim_time', '_force_trim_asked_counter', '_minimum_ask_number_before_force_trim', '_counter_reset_period')
+    
+    
+    def __init__(self, minimum_ask_number_before_force_trim, counter_reset_period):
+        # type: (int, float) -> None
+        self._last_force_trim_time = 0  # type: float
+        self._force_trim_asked_counter = 0  # type: int
+        self._minimum_ask_number_before_force_trim = minimum_ask_number_before_force_trim  # type: int
+        self._counter_reset_period = counter_reset_period  # type: float
+    
+    
+    def ask_for_memory_trim(self, context):
+        # type: (unicode) -> None
+        now = get_now()
+        if self._counter_reset_period > 0:
+            if now - self._last_force_trim_time >= self._counter_reset_period:
+                self._last_force_trim_time = now
+                self._force_trim_asked_counter = 0
+        
+        self._force_trim_asked_counter += 1
+        if self._force_trim_asked_counter >= self._minimum_ask_number_before_force_trim:
+            self._last_force_trim_time = now
+            self._force_trim_asked_counter = 0
+            force_memory_trimming(context)
+
+
 def start_malloc_trim_thread():
     memory_trimming_enable_gc_collect_2()
     t = threading.Thread(None, target=malloc_trim, name='malloc-trim')
@@ -74,10 +125,13 @@ def force_malloc_trim():
     is_malloc_trim_forced = True  # ask the malloc thread to do the job
 
 
-def force_memory_trimming():
+def force_memory_trimming(context):
+    # type: (unicode) -> None
+    gen = 2
+    pid = os.getpid()
     before_collect = time.time()
-    gc.collect(2)
-    logger.log_perf(before_collect, "memory", "forced collect.2 pid:[%s]" % os.getpid(), min_time=0.05)
+    gc.collect(gen)
+    logger.log_perf(before_collect, u'MEMORY', u'REQUESTED python garbage collection ( %s ) for memory area ( generation %d ) in process with pid %d' % (context, gen, pid), min_time=0)
     force_malloc_trim()
 
 
@@ -123,11 +177,11 @@ def malloc_trim(do_forever=True):
                     malloc_trim_limit_nb_call_to_gc_collect_2 = malloc_trim_limit_nb_call_to_gc_collect_2 - 1
                 gen = 2
             gc.collect(gen)
-            logger.log_perf(before_collect, "memory", "collect.%d pid:[%s]" % (gen, pid), min_time=0.05)
+            logger.log_perf(before_collect, u'MEMORY', u'PERIODIC python garbage collection for memory area ( generation %d ) in process with pid %d' % (gen, pid), min_time=0)
             
             time_start = time.time()
             libc6.malloc_trim(0)
-            logger.log_perf(time_start, "memory", "malloc_trim pid[%s]" % pid)
+            logger.log_perf(time_start, u'MEMORY', u'Return of freed memory space to the OS ( malloc_trim ) in process with pid %d' % pid)
             if not do_forever:
                 return
             # Sleep 60s, but maybe launch malloc_trim is ask by another thread
@@ -138,10 +192,10 @@ def malloc_trim(do_forever=True):
                 if is_malloc_trim_forced:
                     time_start = time.time()
                     libc6.malloc_trim(0)
-                    logger.log_perf(time_start, "memory", "malloc_trim pid[%s]" % pid)
+                    logger.log_perf(time_start, u'MEMORY', u'Return of freed memory space to the OS ( malloc_trim ) in process with pid %d' % pid)
                     is_malloc_trim_forced = False
     except Exception:  # noqa
-        logger.info("malloc_trim unsupport by this OS.")
+        logger.info(u'malloc_trim unsupported by this OS.')
 
 
 def get_memory_consumption():
@@ -652,7 +706,7 @@ def __clean_and_uniq(lst):
 
 
 def to_split(val, split_on_coma=True):
-    if isinstance(val, list):
+    if isinstance(val, (list, set)):
         return __clean_and_uniq(val)
     if not split_on_coma:
         return __clean_and_uniq([val])
@@ -1061,14 +1115,15 @@ def apply_change_recursive_pattern_change(s, rule):
     return apply_change_recursive_pattern_change(s, t)
 
 
-def safe_add_to_dict(dict, key, value, as_set=False):
-    if key not in dict:
-        dict[key] = set([value]) if as_set else [value]
+def safe_add_to_dict(dictionary, key, value, as_set=False):
+    # type: (Dict, unicode, Any, bool) -> None
+    if key not in dictionary:
+        dictionary[key] = set([value]) if as_set else [value]
     else:
         if as_set:
-            dict[key].add(value)
+            dictionary[key].add(value)
         else:
-            dict[key].append(value)
+            dictionary[key].append(value)
 
 
 # For service generator, get dict from a _custom properties
@@ -1172,7 +1227,11 @@ def get_key_value_sequence(entry, default_value=None):
                         xy_couples.append((x, y))
                         # We must search if we've gotother X-Y, so
                         # we delete this one, and loop
+                        _old_key = key
                         key = key.replace('[%d-%d]' % (x, y), 'Z' * 10)
+                        if key == _old_key:  # Critical error, abort for now
+                            return None, GET_KEY_VALUE_SEQUENCE_ERROR_NODE
+                        del _old_key
                     else:  # no more X-Y in it
                         still_loop = False
                 
@@ -1255,7 +1314,7 @@ class DuplicateForEachParser(object):
     
     @staticmethod
     def parse(data_value, default_value=None):
-        # type: (str, [str, None]) -> Tuple[List, int]
+        # type: (str, Union[str, None]) -> Tuple[Union[List, None], int]
         if data_value in ('null', '__DEFAULT_NO_TEMPLATE__'):
             return [{'VALUE': None, 'VALUE1': None, 'KEY': data_value}], DuplicateForEachStatus.NOERROR
         
@@ -1273,14 +1332,14 @@ class DuplicateForEachParser(object):
         
         _dfe_elements = DuplicateForEachParser._split_with_indexes(data_value, _indexes_comma)
         
-        checks_list = DuplicateForEachParser._split_key_and_value(_dfe_elements, default_value)
+        checks_list, status = DuplicateForEachParser._split_key_and_value(_dfe_elements, default_value)
         
-        return checks_list, DuplicateForEachStatus.NOERROR
+        return checks_list, status
     
     
     @staticmethod
     def unparse(formatted_dfe, default_value=None):
-        # type: (List[dict], [None, str]) -> str
+        # type: (List[dict], Union[None, str]) -> str
         final_list = []
         DuplicateForEachParser.get_default_value_to_list(default_value)
         
@@ -1295,7 +1354,7 @@ class DuplicateForEachParser(object):
     
     @staticmethod
     def _get_all_indexes(string, key):
-        # type: (str, str) -> list
+        # type: (str, str) -> List
         indexes = []
         last_index = -1
         while True:
@@ -1353,7 +1412,7 @@ class DuplicateForEachParser(object):
     
     @staticmethod
     def _split_key_and_value(dfe_elements, default_value):
-        # type: (list, str) -> list
+        # type: (List, str) -> Tuple[Union[List, None], int]
         
         raw_check_list = []
         for flat_dfe in dfe_elements:
@@ -1394,7 +1453,7 @@ class DuplicateForEachParser(object):
     
     @staticmethod
     def _split_from_xrange_format(check_list):
-        # type: (list) -> list
+        # type: (List) -> Tuple[Union[List, None], int]
         to_return = []
         node_pattern = re.compile(DuplicateForEachParser.REGEX_NODE)
         for check in check_list:
@@ -1417,7 +1476,11 @@ class DuplicateForEachParser(object):
                         xy_couples.append((x, y))
                         # We must search if we've got other X-Y, so
                         # we delete this one, and loop
+                        _old_key = key
                         key = key.replace('[%d-%d]' % (x, y), 'Z' * 10)
+                        if key == _old_key:  # Critical error, abort for now
+                            return None, DuplicateForEachStatus.NODE_ERROR
+                        del _old_key
                     else:  # no more X-Y in it
                         still_loop = False
                 
@@ -1434,7 +1497,7 @@ class DuplicateForEachParser(object):
                         new_r[key] = check[key]
                     new_r['KEY'] = res
                     to_return.append(new_r)
-        return to_return
+        return to_return, DuplicateForEachStatus.NOERROR
     
     
     @staticmethod
@@ -1453,7 +1516,7 @@ def to_float_kb(size):
 
 # ------------ MISC ------------
 def transform_str_key_dict_into_int_dict(d):
-    # type (dict) -> dict
+    # type: (Dict) -> Dict
     new_d = {}
     for (str_k, v) in d.iteritems():
         try:
