@@ -91,7 +91,7 @@ class Worker(object):
         # counter of action give each ticks
         self.action_counter_by_tick = 0
         
-        self.pipe_in_actions_todo_send_by_executor_to_worker, self.pipe_out_actions_todo_send_by_executor_to_worker = build_shinken_queue(u'Main>W[%s]' % self.id, u'W[%s]>Main' % self.id)
+        self.pipe_in_actions_todo_send_by_executor_to_worker, self.pipe_out_actions_todo_send_by_executor_to_worker = build_shinken_queue(u'', u'Main', u'W[%s]' % self.id)
         
         self.worker_type = worker_type
         
@@ -99,7 +99,7 @@ class Worker(object):
         if os.name == 'nt':
             self.http_daemon = None
             # sockets cannot be pickled, we have to use multiprocessing.queue We need one queue for each way of communication to avoid reading data we have just sent
-            self.pipe_in_actions_done_by_worker_to_send_at_executor, self.pipe_out_actions_done_by_worker_to_send_at_executor = build_shinken_queue(u'Main>W[%s]' % self.id, u'W[%s]>Main' % self.id)
+            self.pipe_in_actions_done_by_worker_to_send_at_executor, self.pipe_out_actions_done_by_worker_to_send_at_executor = build_shinken_queue(u'DONE', u'Main', u'W[%s]' % self.id)
         else:
             self.http_daemon = http_daemon
             # each way of communication has its own buffers, just use them, no risk of interference
@@ -553,6 +553,9 @@ class Worker(object):
             
             self.last_loop_time.value = time.time()
             
+            if wait_time > 0.1:
+                wait_time = 0.1
+            
             # Little sleep
             time.sleep(wait_time)
             
@@ -562,3 +565,5 @@ class Worker(object):
                 if wait_for_fix_cycle < 0.1:  # only log if we need
                     self.logger.debug(u'Sleep time: %.3fs' % wait_for_fix_cycle)
                 time.sleep(wait_for_fix_cycle)
+            
+            self.last_loop_time.value = time.time()
