@@ -1,15 +1,18 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2013-2018:
+# Copyright (C) 2013-2022:
 # This file is part of Shinken Enterprise, all rights reserved.
-import datetime
+
 import json
 import os
 import re
 import threading
-import time
 from collections import OrderedDict, namedtuple
+from weakref import proxy as weak_ref
+
+import datetime
+import time
 
 from shinken.exceptions.business import ShinkenExceptionValueError, ShinkenExceptionKeyError
 from shinken.log import logger
@@ -24,14 +27,15 @@ except ImportError:
     from io import StringIO
 
 if TYPE_CHECKING:
-    from shinken.misc.type_hint import Dict, Optional, List, Collection, Set, Union
+    from shinken.misc.type_hint import Dict, Optional, List, Collection, Set, Union, Tuple
     from shinken.objects.contact import Contact
-    from shinken.objects.host import Host
-    from shinken.objects.service import Service
+    from shinken.objects.broker.broker_host import BrokerHost
+    from shinken.objects.broker.broker_service import BrokerService
     from shinken.webui.bottlewebui import MultiDict
     from shinken.misc.monitoring_item_manager.monitoring_item_manager import MonitoringItemManager
+    from shinken.objects.schedulingitem import SchedulingItem
 
-# ## Will be populated by the UI with it's own value
+# ## Will be populated by the UI with its own value
 FRONTEND_DATE_FORMAT = '%Y-%m-%d %H:%M:%S'
 DEFAULT_SORT = {'tag': 'host_name', 'value': 'asc'}
 
@@ -44,16 +48,16 @@ class DateFilterOperator(object):
 
 
 class StaticCache(object):
-    lock = threading.RLock()
-    part_configuration_id = -1
-    hosts_by_name = {}  # type: Dict[unicode,Host]
-    hosts_by_uuid = {}  # type: Dict[unicode,Host]
-    checks_by_uuid = {}  # type: Dict[unicode,Service]
-    checks_by_name = {}  # # type: Dict[unicode,List[Service]]
+    configuration_uuid = -1
+    hosts_by_name = {}  # type: Dict[unicode,BrokerHost]
+    hosts_by_uuid = {}  # type: Dict[unicode,BrokerHost]
+    checks_by_uuid = {}  # type: Dict[unicode,BrokerService]
+    checks_by_name = {}  # type: Dict[unicode,List[BrokerService]]
     
     
     @staticmethod
     def _init_static_cache(item):
+        # type: (Union[BrokerHost,BrokerService]) -> None
         item_type = 'host'
         extended_item_type = item_type
         if item.__class__.my_type == 'service':
@@ -145,34 +149,36 @@ class StaticCache(object):
     
     
     @classmethod
-    def refresh(cls, items, part_configuration_id):
-        with cls.lock:
-            if cls.part_configuration_id != part_configuration_id:
-                cls.hosts_by_name = {}
-                cls.hosts_by_uuid = {}
-                cls.checks_by_name = {}
-                cls.checks_by_uuid = {}
-                for i in items:
-                    cls._init_static_cache(i)
-                    
-                    name = i.list_cache_static[u'name'].lower()
-                    uuid = i.list_cache_static[u'uuid'].lower()
-                    if i.list_cache_static[u'type'] == u'check':
-                        cls.checks_by_uuid[uuid] = i
-                        if name in cls.checks_by_name:
-                            cls.checks_by_name[name].append(i)
-                        else:
-                            cls.checks_by_name[name] = [i]
-                    else:
-                        cls.hosts_by_name[name] = i
-                        cls.hosts_by_uuid[uuid] = i
-                
-                cls.part_configuration_id = part_configuration_id
+    def refresh(cls, items, configuration_uuid):
+        # type: (List[Union[BrokerHost, BrokerService]], unicode) -> None
+        
+        if cls.configuration_uuid != configuration_uuid:
+            # Reset cache only on new configuration_uuid
+            cls.hosts_by_name = {}
+            cls.hosts_by_uuid = {}
+            cls.checks_by_name = {}
+            cls.checks_by_uuid = {}
+            cls.configuration_uuid = configuration_uuid
+        
+        for i in items:
+            cls._init_static_cache(i)
+            item_static_cache = getattr(i, u'list_cache_static')
+            name = item_static_cache[u'name'].lower()
+            uuid = item_static_cache[u'uuid'].lower()
+            if item_static_cache[u'type'] == u'check':
+                cls.checks_by_uuid[uuid] = i
+                if name in cls.checks_by_name:
+                    cls.checks_by_name[name].append(weak_ref(i))
+                else:
+                    cls.checks_by_name[name] = [weak_ref(i)]
+            else:
+                cls.hosts_by_name[name] = weak_ref(i)
+                cls.hosts_by_uuid[uuid] = weak_ref(i)
     
     
     @classmethod
     def get_host_by_uuid(cls, uuid, user=None):
-        # type: (unicode, Optional[Contact]) -> Optional[Host]
+        # type: (unicode, Optional[Contact]) -> Optional[BrokerHost]
         if user and not check_user_item_access(user, uuid):
             return None
         if uuid in cls.hosts_by_uuid:
@@ -182,7 +188,7 @@ class StaticCache(object):
     
     @classmethod
     def get_check_by_uuid(cls, uuid, user=None):
-        # type: (unicode, Optional[Contact]) -> Optional[Service]
+        # type: (unicode, Optional[Contact]) -> Optional[BrokerService]
         if user and not check_user_item_access(user, uuid):
             return None
         if uuid in cls.checks_by_uuid:
@@ -192,7 +198,7 @@ class StaticCache(object):
     
     @classmethod
     def get_host_by_name(cls, name, user=None):
-        # type: (unicode, Optional[Contact]) -> Optional[Host]
+        # type: (unicode, Optional[Contact]) -> Optional[BrokerHost]
         if name in cls.hosts_by_name:
             host = cls.hosts_by_name[name]
             if not user or check_user_item_access(user, host.get_uuid()):
@@ -202,7 +208,7 @@ class StaticCache(object):
     
     @classmethod
     def get_check_by_name(cls, host, name, user=None):
-        # type: (Host, unicode, Optional[Contact]) -> Optional[Service]
+        # type: (BrokerHost, unicode, Optional[Contact]) -> Optional[BrokerService]
         if not host or not name:
             return None
         host_uuid = host.get_uuid()
@@ -231,16 +237,16 @@ def _get_state(item):
 
 # TODO put in helper after patch
 def _get_context(elt):
-    # type: (Union[Host,Service]) -> (unicode, unicode)
+    # type: (SchedulingItem) -> Tuple[unicode, unicode]
     class_name = elt.__class__.my_type
     is_host = (class_name == 'host')
     is_service = (class_name == 'service')
     context = 'NOTHING', 'NOTHING'
     
-    # DISABLED is for hosts, service can't be like this because an host disabled got no service
+    # DISABLED is for hosts, service can't be like this because a host disabled got no service
     if is_host and 'disabled' in elt.tags:
         context = 'DISABLED', 'DISABLED'
-    elif elt.downtimes and next((dt for dt in elt.downtimes if dt.ref == elt and dt.is_in_effect), None):
+    elif elt.downtimes and next((dt for dt in elt.downtimes if dt.ref.get_instance_uuid() == elt.get_instance_uuid() and dt.is_in_effect), None):
         context = 'DOWNTIME', 'DOWNTIME'
     elif elt.is_in_inherited_downtime():
         context = 'INHERITED-DOWNTIME', 'DOWNTIME'
@@ -318,7 +324,7 @@ def _build_var_sort_key(item, tag):
     return [int(s) if s.isdigit() else s.lower() for s in re.split(r'(\d+)', _value.encode('utf-8', 'ignore') if hasattr(_value, 'encode') else str(_value))]
 
 
-def _sort_list_api_part_all(sorts, items):
+def sort_list_api_part_all(sorts, items):
     if not sorts:
         sorts = [DEFAULT_SORT]
     for sort in sorts:
@@ -327,24 +333,27 @@ def _sort_list_api_part_all(sorts, items):
         items.sort(key=lambda item: item.sort_cache_static[tag] if tag in item.sort_cache_static else _build_var_sort_key(item, tag), reverse=reverse)
 
 
-def _list_all_values(items):
+def list_all_values(items):
     all_values = {
-        'host_templates': set(),
-        'realm'         : set(),
-        'host_groups'   : set(),
-        'contact_groups': set(),
-        'contacts'      : set(),
+        'host_templates'     : set(),
+        'realm'              : set(),
+        'host_groups'        : set(),
+        'contact_groups'     : set(),
+        'contacts'           : set(),
+        'notification_period': set(),
     }
     update_realm = all_values['realm'].add
     update_host_template = all_values['host_templates'].update
     update_host_group = all_values['host_groups'].update
     update_contact_group = all_values['contact_groups'].update
     update_contacts = all_values['contacts'].update
+    update_notification_period = all_values['notification_period'].add
     
     for item in items:
         list_cache_static = item.list_cache_static
         update_contact_group(list_cache_static['contact_groups'])
         update_contacts(list_cache_static['contacts'])
+        update_notification_period(list_cache_static['notification_period'])
         if not list_cache_static['type'] == 'check':
             update_host_template(list_cache_static['host_templates'])
             update_host_group(list_cache_static['host_groups'])
@@ -441,7 +450,8 @@ class FilterItem(object):
             raise (NotImplementedError(u'monitoring_item_manager not available'))
     
     
-    def _translate_filter_value(self, filter_tag, filter_value, only_allowed_values=None):
+    @classmethod
+    def _translate_filter_value(cls, filter_tag, filter_value, only_allowed_values=None):
         filter_value = [v.lower() for v in filter_value]
         
         if filter_tag == u'extended_type' and u'check' in filter_value:
@@ -475,7 +485,7 @@ class FilterItem(object):
             if wrong_values:
                 raise ShinkenExceptionValueError(text=u'wrong value%s %s' % (u's' if len(wrong_values) > 1 else u'', wrong_values))
         
-        compare_type = self.FILTER_TYPE[filter_tag]
+        compare_type = cls.FILTER_TYPE[filter_tag]
         if compare_type == u'int':
             filter_value = set([int(v) for v in filter_value])
         elif compare_type == u'bool':
@@ -494,14 +504,15 @@ class FilterItem(object):
                     delay = int(delay)
                 except ValueError as e:
                     raise ShinkenExceptionValueError(text=u'[ %s ] => %s' % (operator, str(e)))
-                operator = DateFilterOperator.NEWER if operator == u'latest' or operator == u'in_more_than' or operator == u'in-less-than' or operator == u'later-than' else DateFilterOperator.OLDER
+                operator = DateFilterOperator.NEWER if operator == u'latest' or operator == u'in_more_than' or operator == u'in-more-than' or operator == u'later-than' else DateFilterOperator.OLDER
                 filter_value = [DateFilter(operator, delay)]
             else:
                 raise ShinkenExceptionKeyError(text=u'unknown date constraint [ %s ]' % val)
         return filter_value
     
     
-    def _parse_filter(self, get_filters, allowed_filter=None, only_allowed_values=None, default_if_empty=None):
+    @classmethod
+    def parse_filter(cls, get_filters, allowed_filter=None, only_allowed_values=None, default_if_empty=None, filter_item_instance=None):
         filters = []
         cpt = - 1
         for get_filter in get_filters:
@@ -514,8 +525,8 @@ class FilterItem(object):
                 if allowed_filter:
                     if filter_tag not in allowed_filter:
                         error_text = u'filtering[%s]: invalid field name [ %s ]' % (cpt, filter_tag_name)
-                        if hasattr(self, u'ALLOWED_FILTER'):
-                            error_text = u'filtering[%s]: field name [ %s ] not allowed in this route ' % (cpt, filter_tag_name) if filter_tag in getattr(self, u'ALLOWED_FILTER') else error_text
+                        if filter_item_instance and hasattr(filter_item_instance, u'ALLOWED_FILTER'):
+                            error_text = u'filtering[%s]: field name [ %s ] not allowed in this route ' % (cpt, filter_tag_name) if filter_tag in getattr(filter_item_instance, u'ALLOWED_FILTER') else error_text
                         raise ShinkenExceptionKeyError(text=error_text)
                     filter_tag = allowed_filter[filter_tag]
                 if len(split) > 1:
@@ -528,7 +539,7 @@ class FilterItem(object):
                     raise ShinkenExceptionValueError(text=u'filtering[%s]: missing value for field [ %s ]' % (cpt, filter_tag_name))
                 
                 try:
-                    filter_value = self._translate_filter_value(filter_tag, filter_value, only_allowed_values)
+                    filter_value = cls._translate_filter_value(filter_tag, filter_value, only_allowed_values)
                 except (ValueError, ShinkenExceptionValueError) as e:
                     raise ShinkenExceptionValueError(text=u'filtering[%s]: field [ %s ] => %s' % (cpt, filter_tag_name, str(e)))
                 except ShinkenExceptionKeyError as e:
@@ -542,22 +553,24 @@ class FilterItem(object):
                 extra_default_filter = {}
                 for k in default_if_empty:
                     if k not in _filter:
-                        extra_default_filter.update({k: self._translate_filter_value(k, default_if_empty[k])})
+                        extra_default_filter.update({k: cls._translate_filter_value(k, default_if_empty[k])})
                 if extra_default_filter:
                     _filter.update(extra_default_filter)
             filters.append(_filter)
         return filters
     
     
-    def _filter_root_problems(self, filters, items):
-        items = self._filter_elements_with_search(items, [{'is_root_problem': set(['true'])}])
+    @classmethod
+    def filter_root_problems(cls, filters, items):
+        items = cls.filter_elements_with_search(items, [{'is_root_problem': set(['true'])}])
         # remove the occurrence of is_root_problem in filter
         for _filter in filters[:]:
             _filter.pop('is_root_problem', None)
         return items
     
     
-    def _filter_elements_with_search(self, items, _filters, counters=None):
+    @classmethod
+    def filter_elements_with_search(cls, items, _filters, counters=None):
         new_elements = []
         # counter on found elements, with this field matching :
         # 0 hosts found,
@@ -578,7 +591,7 @@ class FilterItem(object):
                 type_found = 2
             
             for _filter in _filters:
-                if self._apply_filter(_filter, item):
+                if cls._apply_filter(_filter, item):
                     add_element = True
                     break
             
@@ -591,7 +604,7 @@ class FilterItem(object):
     
     # Yes this function is ugly but do not refactor without check performance first
     @staticmethod
-    def _get_value_for_filter(item, filter_name):
+    def get_value_for_filter(item, filter_name):
         if filter_name in item.filter_cache_static:
             return item.filter_cache_static[filter_name]
         
@@ -623,11 +636,14 @@ class FilterItem(object):
             return item.perf_data.lower()
     
     
-    def _apply_filter(self, _filter, item):
+    @classmethod
+    def _apply_filter(cls, _filter, item):
+        # Yes this function is ugly but do not refactor without check performance first
+        
         match_filter = True
         for (filter_tag, filter_values) in _filter.iteritems():
-            value = self._get_value_for_filter(item, filter_tag)
-            compare_type = self.FILTER_TYPE[filter_tag]
+            value = cls.get_value_for_filter(item, filter_tag)
+            compare_type = cls.FILTER_TYPE[filter_tag]
             # 0 is a value
             if value != 0 and not value:
                 match_filter = False
@@ -644,11 +660,18 @@ class FilterItem(object):
                 _match_tag = True in [v in value for v in filter_values]
             elif compare_type == 'date':
                 filter_value = filter_values[0]
+                # SEF-10916 For the "next_check" property, the visu adds a "-" to the delay because it's in the future,
+                # so we force the negativity to be compatible with modules that use filters (ex: broker-module-livedata v2)
                 if isinstance(filter_value, DateFilter):
-                    if filter_value.operator == DateFilterOperator.NEWER:
-                        _match_tag = filter_value.delay >= abs(time.time() - value)
+                    if filter_tag == u'next_check':
+                        delay = -abs(filter_value.delay)
                     else:
-                        _match_tag = filter_value.delay <= abs(time.time() - value)
+                        delay = filter_value.delay
+                    
+                    if filter_value.operator == DateFilterOperator.NEWER:
+                        _match_tag = delay >= time.time() - value
+                    else:
+                        _match_tag = delay <= time.time() - value
                 else:
                     value = datetime.datetime.fromtimestamp(value).strftime(FRONTEND_DATE_FORMAT)
                     _match_tag = filter_value in value
@@ -663,39 +686,44 @@ class FilterItem(object):
     
     
     @staticmethod
-    def _api_list_element_dict(item):
-        # type: (Union[Host,Service]) -> Dict
-        list_cache_var = getattr(item, 'list_cache_var', {})
+    def api_list_element_dict(item):
+        # type: (Union[BrokerHost,BrokerService]) -> Dict
+        _new_cache_var = {}
+        list_cache_var = getattr(item, u'list_cache_var', {})
         if not list_cache_var:
-            list_cache_var['business_impact'] = item.business_impact
-            list_cache_var['status'] = _get_state(item)
-            list_cache_var['context'], _ = _get_context(item)
-            list_cache_var['status_since'] = int(item.last_state_change)
-            list_cache_var['last_hard_state_change'] = int(item.last_hard_state_change if item.state_type == 'HARD' else 0)
-            list_cache_var['is_status_confirmed'] = (item.state_type == 'HARD')
-            list_cache_var['attempts'] = item.attempt
-            list_cache_var['max_attempts'] = item.max_check_attempts
-            list_cache_var['is_root_problem'] = item.is_problem
-            list_cache_var['output'] = item.output
-            list_cache_var['long_output'] = item.long_output
-            list_cache_var['perf_data'] = item.perf_data
-            list_cache_var['smart_perf_data'] = PerfDatas(item.perf_data).as_list()
-            list_cache_var['got_business_rule'] = item.got_business_rule
-            list_cache_var['in_flapping'] = item.is_flapping
-            list_cache_var['inherited_flapping'] = item.inherited_flapping
-            list_cache_var['in_partial_flapping'] = item.is_partial_flapping
-            list_cache_var['notes_multi_url'] = item.notes_multi_url
-            list_cache_var['active_checks_enabled'] = item.active_checks_enabled
-            
-            get_ack_on_item(item, list_cache_var)
-            get_downtime_on_item(item, list_cache_var)
+            _context, ___ = _get_context(item)
+            _new_cache_var = {
+                u'business_impact'       : item.business_impact,
+                u'status'                : _get_state(item),
+                u'context'               : _context,
+                u'status_since'          : int(item.last_state_change),
+                u'last_hard_state_change': int(item.last_hard_state_change if item.state_type == u'HARD' else 0),
+                u'is_status_confirmed'   : (item.state_type == u'HARD'),
+                u'attempts'              : item.attempt,
+                u'max_attempts'          : item.max_check_attempts,
+                u'is_root_problem'       : item.is_problem,
+                u'output'                : item.output,
+                u'long_output'           : item.long_output,
+                u'perf_data'             : item.perf_data,
+                u'smart_perf_data'       : PerfDatas(item.perf_data).as_list(),
+                u'got_business_rule'     : item.got_business_rule,
+                u'in_flapping'           : item.is_flapping,
+                u'inherited_flapping'    : item.inherited_flapping,
+                u'in_partial_flapping'   : item.is_partial_flapping,
+                u'notes_multi_url'       : item.notes_multi_url,
+                u'notes_url'             : item.notes_url,
+                u'active_checks_enabled' : item.active_checks_enabled,
+            }
+            get_ack_on_item(item, _new_cache_var)
+            get_downtime_on_item(item, _new_cache_var)
             
             if not item.got_business_rule:
-                list_cache_var['next_check'] = item.next_chk
-                list_cache_var['last_check'] = item.last_chk
+                _new_cache_var[u'next_check'] = item.next_chk
+                _new_cache_var[u'last_check'] = item.last_chk
             
-            list_cache_var.update(item.list_cache_static)
-            item.list_cache_var = list_cache_var
+            _new_cache_var.update(getattr(item, u'list_cache_static', {}))
+            item.list_cache_var = _new_cache_var
+            return _new_cache_var
         
         return list_cache_var
     
@@ -809,7 +837,7 @@ class FilterItem(object):
             self.full_filter_for_log = get_filters[:]
         
         if get_filters:
-            self.filters = self._parse_filter(get_filters, allowed_filters, only_allowed_values, default_if_empty)
+            self.filters = self.parse_filter(get_filters, allowed_filters, only_allowed_values, default_if_empty, filter_item_instance=self)
         elif default_if_empty:
             # self.filters = [{k: self._translate_filter_value(k, default_if_empty[k]) for k in default_if_empty}]
             self.filters = {}
@@ -847,7 +875,7 @@ class FilterItem(object):
         start_time = time.time()
         initial_set = initial_set.values()
         if (self.sorts and self.sorts != [DEFAULT_SORT]) or self.filters:
-            _sort_list_api_part_all(self.sorts, initial_set)
+            sort_list_api_part_all(self.sorts, initial_set)
         logger.log_perf(start_time, u'get_list_api_part_all', u'sort %s' % log_name)
         
         if not isinstance(counters, list) or len(counters) != 3:
@@ -873,7 +901,7 @@ class FilterItem(object):
                 if self.page is None or self.page == page_counters.current_page:
                     counters[2] = counters[2] + len(check_for_host)
                     if self.sorts and self.sorts != [DEFAULT_SORT]:
-                        _sort_list_api_part_all(self.sorts, check_for_host)
+                        sort_list_api_part_all(self.sorts, check_for_host)
                     if kept_checks is not None:
                         kept_checks.add(host.host_name, check_for_host)
                     else:
@@ -916,7 +944,7 @@ class FilterItem(object):
             output_string.write(u'"%s":[' % item_name)
             
             for output_element in host_elements:
-                output_element = self._api_list_element_dict(output_element)
+                output_element = self.api_list_element_dict(output_element)
                 host_name = output_element[u'host_name']
                 if self.ALLOWED_OUTPUT:
                     # output_element = {self.ALLOWED_OUTPUT[k]: output_element[k] for k in self.ALLOWED_OUTPUT if k in output_element}
@@ -936,7 +964,7 @@ class FilterItem(object):
                     if host_name in check_elements:
                         checks = check_elements[host_name]
                         for output_check in checks:
-                            output_check = self._api_list_element_dict(output_check)
+                            output_check = self.api_list_element_dict(output_check)
                             if self.ALLOWED_OUTPUT:
                                 # output_check = {self.ALLOWED_OUTPUT[k]: output_check[k] for k in self.ALLOWED_OUTPUT if k in output_check}
                                 tmp_output = {}
@@ -991,19 +1019,13 @@ class FilterItem(object):
         items = self.monitoring_item_manager.get_all_hosts_and_services()
         
         configuration_id = u''
-        part_configuration_id = u''
         last_part_configuration_incarnation = self.monitoring_item_manager.get_last_part_configuration_incarnation()
         if last_part_configuration_incarnation:
             configuration_id = last_part_configuration_incarnation.get_uuid()
-            part_configuration_id = u'%s-%s' % (last_part_configuration_incarnation.get_uuid(), last_part_configuration_incarnation.get_part_id())
         full_nb_items_in_broker = len(items)
         
         logger.log_perf(start_time, log_prefix, u'get_all_hosts_and_services len [%s]' % full_nb_items_in_broker)
         
-        start_time = time.time()
-        StaticCache.refresh(items, part_configuration_id)
-        logger.log_perf(start_time, log_prefix, u'_init_static_cache len [%s]' % full_nb_items_in_broker)
-        
         # Filter with authorization
         # TODO mark filter elements.
         if user:
@@ -1020,8 +1042,8 @@ class FilterItem(object):
         all_values = {}
         if self.result_set_id is not None and self.result_set_id != configuration_id:
             start_time = time.time()
-            all_values = _list_all_values(items)
-            logger.log_perf(start_time, log_prefix, u'_list_all_values')
+            all_values = list_all_values(items)
+            logger.log_perf(start_time, log_prefix, u'list_all_values')
         
         start_time = time.time()
         hosts = {}
@@ -1051,11 +1073,11 @@ class FilterItem(object):
         if self.filters:
             filters_contain_root_problems = u'true' in self.filters[0].get(u'is_root_problem', set([u'false']))
             if filters_contain_root_problems:
-                items = self._filter_root_problems(self.filters, items)
+                items = self.filter_root_problems(self.filters, items)
             
             # normal filters
             nb_found = [0, 0, 0]
-            items = self._filter_elements_with_search(items, self.filters, nb_found)
+            items = self.filter_elements_with_search(items, self.filters, nb_found)
             nb_hosts_found = nb_found[0]
             nb_clusters_found = nb_found[1]
             nb_checks_found = nb_found[2]
@@ -1063,7 +1085,7 @@ class FilterItem(object):
             nb_hosts_found = nb_hosts_total
             nb_clusters_found = nb_clusters_total
             nb_checks_found = nb_checks_total
-        logger.log_perf(start_time, log_prefix, u'_filter_elements_with_search ')
+        logger.log_perf(start_time, log_prefix, u'filter_elements_with_search ')
         
         nb_items_post_filter = len(items)
         
@@ -1078,7 +1100,7 @@ class FilterItem(object):
         if self.flattened_view:
             start_time = time.time()
             if (self.sorts and self.sorts != [DEFAULT_SORT]) or self.filters:
-                _sort_list_api_part_all(self.sorts, items)
+                sort_list_api_part_all(self.sorts, items)
             logger.log_perf(start_time, log_prefix, u'sort all (flattened view)')
             
             start_time = time.time()
@@ -1416,10 +1438,10 @@ class FilterItem(object):
     
     
     def _get_elements_with_allowed_output(self, elements, fathers_checks=None):
-        # type: (List[Union[Host,Service]], Dict) -> List
+        # type: (List[Union[BrokerHost,BrokerService]], Dict) -> List
         updated_fathers = []
         for output_element in elements:
-            output_element = self._api_list_element_dict(output_element)
+            output_element = self.api_list_element_dict(output_element)
             host_name = output_element[u'host_name']
             output_element = self._update_element_output(output_element, [])
             updated_fathers.append(output_element)
@@ -1429,7 +1451,7 @@ class FilterItem(object):
             if host_name in fathers_checks:
                 checks = fathers_checks[host_name]
                 for output_check in checks:
-                    output_check = self._update_element_output(self._api_list_element_dict(output_check), [u'uuid_parent', u'host_name'])
+                    output_check = self._update_element_output(self.api_list_element_dict(output_check), [u'uuid_parent', u'host_name'])
                     if u'checks' not in output_element and output_check:
                         output_element[u'checks'] = []
                     output_element[u'checks'].append(output_check)
