@@ -15,7 +15,7 @@ SHINKEN.TOOLS.TEST_AUTO_LIST_ALL = (function ( self ) {
     self.DOM.getMassActionOptions                           = function () {
         return self.DOM.getMassActionContainer().querySelectorAll( ".shinken-option" );
     };
-    self.DOM.getMassActionButton                           = function () {
+    self.DOM.getMassActionButton                            = function () {
         return document.getElementById( 'btn-action-launch' );
     };
     self.DOM.getFirstActiveMassActionOption                 = function ( option_name ) {
@@ -230,7 +230,7 @@ SHINKEN.TOOLS.TEST_AUTO_LIST_ALL = (function ( self ) {
         return false;
     };
     self.ACTION.selectMassAction                            = function ( timeout, mass_action_name ) {
-        var _to_return = 3000;
+        var _to_return = 500;
         var _uuid      = SHINKEN.TOOLS.TEST_AUTO_LOGGER.add( timeout, 'selectMassAction', mass_action_name );
         setTimeout( function () {
             SHINKEN.TOOLS.TEST_AUTO_LOGGER.changeElementStatus( _uuid, "IN_PROGRESS" );
@@ -258,14 +258,16 @@ SHINKEN.TOOLS.TEST_AUTO_LIST_ALL = (function ( self ) {
     };
     self.ACTION.clickOnExecuteMassAction                    = function ( timeout, skip_do_next ) {
         setTimeout( function () {
-            SHINKEN.HIGHWAY.changeUrl( SHINKEN.TOOLS.TEST_AUTO_PHASE.getNextPhaseLink() );
-            SHINKEN.TOOLS.TEST_AUTO_DOM.click( 0,  self.DOM.getMassActionButton() );
-        }, timeout  );
-        return 5000;
+            if ( !skip_do_next ) {
+                SHINKEN.HIGHWAY.changeUrl( SHINKEN.TOOLS.TEST_AUTO_PHASE.getNextPhaseLink() );
+            }
+            SHINKEN.TOOLS.TEST_AUTO_DOM.click( 0, self.DOM.getMassActionButton() );
+        }, timeout );
+        return 3000;
     };
     self.ACTION.clickOnExecuteMassActionWithComment         = function ( timeout ) {
         setTimeout( function () {
-            SHINKEN.TOOLS.TEST_AUTO_DOM.click( 0,  self.DOM.getMassActionButton() );
+            SHINKEN.TOOLS.TEST_AUTO_DOM.click( 0, self.DOM.getMassActionButton() );
             self.DOM.getMassActionButton().click();
         }, timeout );
         return 1000;
@@ -433,7 +435,7 @@ SHINKEN.TOOLS.TEST_AUTO_LIST_ALL = (function ( self ) {
     };
     self.TEST.labelColumnHeader                             = function ( timeout, property_name, label ) {
         var _to_return = 50;
-        var _uuid      = SHINKEN.TOOLS.TEST_AUTO_LOGGER.add( timeout, 'TEST.labelPropertyProposedPopup', label );
+        var _uuid      = SHINKEN.TOOLS.TEST_AUTO_LOGGER.add( timeout, 'TEST.labelColumnHeader', label );
         setTimeout( function () {
             SHINKEN.TOOLS.TEST_AUTO_LOGGER.changeElementStatus( _uuid, "IN_PROGRESS" );
             SHINKEN.TOOLS.TEST_AUTO.__isInnerTextEqualTo( self.DOM.columnHeaderName( property_name ), label );
