@@ -619,7 +619,11 @@ SHINKEN.TOOLS.TEST_AUTO_WEATHER_RUN = (function ( self ) {
             case "00217":
                 switch ( __phase ) {
                     case 1:
-                        _timeout += SHINKEN.TOOLS.TEST_AUTO.changeUser( "test_auto_user_executor", "test_auto_user_executor" );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_SHINKEN_ELEMENT.deleteElement( _timeout, OBJECT.ShinkenElementFactory( { "_id": "00000000000000000000000000000001", "type": OBJECT.CONST.ELEMENT.TYPE.HOSTS } ), "", true ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_SOURCE.ACTION.switchSourceActivity( _timeout, SOURCE__NAME__TEST_AUTO ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_SHINKEN_ELEMENT.ACTION.save_external_shinken_element( _timeout, [200, 202] ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_SOURCE.ACTION.switchSourceActivity( _timeout, SOURCE__NAME__TEST_AUTO, true ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO.changeUser( "test_auto_user_executor", "test_auto_user_executor" ) );
                         break;
                     case 2:
                         ____( SHINKEN.TOOLS.TEST_AUTO.TEST.isConfigEveryone( _timeout ) );
@@ -679,32 +683,31 @@ SHINKEN.TOOLS.TEST_AUTO_WEATHER_RUN = (function ( self ) {
                     case 1:
                         // ALL TRUE
                         _timeout += 2000;
-                        //____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.openUserConfiguration( _timeout ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.setValueUserConfigurationProperty( _timeout, "blink", "ON" ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.setValueUserConfigurationProperty( _timeout, "volume", 90 ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_WEATHER_EDIT.ACTION.IFRAME.clickOnButton( _timeout, 'button_mode_edit' ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_WEATHER_EDIT.ACTION.IFRAME.setJSONInForm( _timeout ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_WEATHER_EDIT.ACTION.IFRAME.clickOnButton( _timeout, 'button_save' ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.clickOnSubMenu( _timeout, SHINKEN.SCREEN.TYPE.SERVICE_WEATHER, "menu_expand_views_service_weather_full" ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_PANEL_WEATHER.TEST.notificationState( _timeout, "TEST-AUTO-00231", true, true ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_PANELS.ACTION.closePanelsModal( _timeout ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_WEATHER.TEST.IFRAME.displayState( _timeout, "sound_notification_container", SHINKEN.TOOLS.TEST_AUTO_WEATHER.CONST.STATE_DISPLAY.VISIBLE ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_WEATHER.TEST.IFRAME.displayState( _timeout, "blink_notification_container", SHINKEN.TOOLS.TEST_AUTO_WEATHER.CONST.STATE_DISPLAY.VISIBLE ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.openUserConfiguration( _timeout ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.setValueUserConfigurationProperty( _timeout, "blink", "ON" ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.setValueUserConfigurationProperty( _timeout, "volume", 90 ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_WEATHER_EDIT.ACTION.IFRAME.clickOnButton( _timeout, 'button_mode_edit' ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_WEATHER_EDIT.ACTION.IFRAME.setJSONInForm( _timeout ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_WEATHER_EDIT.ACTION.IFRAME.clickOnButton( _timeout, 'button_save' ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.clickOnSubMenu( _timeout, SHINKEN.SCREEN.TYPE.SERVICE_WEATHER, "menu_expand_views_service_weather_full" ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_PANEL_WEATHER.TEST.notificationState( _timeout, "TEST-AUTO-00231", true, true ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_PANELS.ACTION.closePanelsModal( _timeout ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_WEATHER.TEST.IFRAME.displayState( _timeout, "sound_notification_container", SHINKEN.TOOLS.TEST_AUTO_WEATHER.CONST.STATE_DISPLAY.VISIBLE ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_WEATHER.TEST.IFRAME.displayState( _timeout, "blink_notification_container", SHINKEN.TOOLS.TEST_AUTO_WEATHER.CONST.STATE_DISPLAY.VISIBLE ) );
                         ____( SHINKEN.TOOLS.TEST_AUTO_WEATHER_EDIT.ACTION.IFRAME.clickOnButton( _timeout, 'button_external_links' ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_WEATHER_EDIT.ACTION.IFRAME.clickOnButton( _timeout, 'popup_external_links' ) );
-                        ____( SHINKEN.TOOLS.TEST_AUTO_WEATHER_EDIT.ACTION.IFRAME.mouseEnterLink( _timeout, "002311" ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_WEATHER_EDIT.ACTION.IFRAME.clickOnButton( _timeout, 'popup_external_links' ) );
+                        //____( SHINKEN.TOOLS.TEST_AUTO_WEATHER_EDIT.ACTION.IFRAME.mouseEnterLink( _timeout, "002311" ) ); //DOESNT WORK
                         //____( SHINKEN.TOOLS.TEST_AUTO_WEATHER_EDIT.TEST.EXTERNAL_LINKS.tooltipOfLinkContainsText( _timeout, "Son : Activé", true  ) );
                         
-                        //____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.openUserConfiguration( _timeout ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.setValueUserConfigurationProperty( _timeout, "blink", "OFF" ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.setValueUserConfigurationProperty( _timeout, "volume", 0 ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.openUserConfiguration( _timeout ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.clickOnSubMenu( _timeout, SHINKEN.SCREEN.TYPE.SERVICE_WEATHER, "menu_expand_views_service_weather_full" ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_PANEL_WEATHER.TEST.notificationState( _timeout, "TEST-AUTO-00231", true, true ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_PANELS.ACTION.closePanelsModal( _timeout ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_WEATHER.TEST.IFRAME.displayState( _timeout, "sound_notification_container", SHINKEN.TOOLS.TEST_AUTO_WEATHER.CONST.STATE_DISPLAY.VISIBLE_AND_DISABLED ) );
-                        //____( SHINKEN.TOOLS.TEST_AUTO_WEATHER.TEST.IFRAME.displayState( _timeout, "blink_notification_container", SHINKEN.TOOLS.TEST_AUTO_WEATHER.CONST.STATE_DISPLAY.VISIBLE_AND_CHILDREN_DISABLED ) );
-                        __do_next = false;
+                        ____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.openUserConfiguration( _timeout ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.setValueUserConfigurationProperty( _timeout, "blink", "OFF" ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.setValueUserConfigurationProperty( _timeout, "volume", 0 ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.openUserConfiguration( _timeout ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_MENU.ACTION.clickOnSubMenu( _timeout, SHINKEN.SCREEN.TYPE.SERVICE_WEATHER, "menu_expand_views_service_weather_full" ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_PANEL_WEATHER.TEST.notificationState( _timeout, "TEST-AUTO-00231", true, true ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_UI_VISU_PANELS.ACTION.closePanelsModal( _timeout ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_WEATHER.TEST.IFRAME.displayState( _timeout, "sound_notification_container", SHINKEN.TOOLS.TEST_AUTO_WEATHER.CONST.STATE_DISPLAY.VISIBLE_AND_DISABLED ) );
+                        ____( SHINKEN.TOOLS.TEST_AUTO_WEATHER.TEST.IFRAME.displayState( _timeout, "blink_notification_container", SHINKEN.TOOLS.TEST_AUTO_WEATHER.CONST.STATE_DISPLAY.VISIBLE_AND_CHILDREN_DISABLED ) );
                         break;
                     case 2:
                         _timeout += 2000;
