@@ -31,15 +31,18 @@ SHINKEN.TOOLS.TEST_AUTO       = (function ( self ) {
         }
         _messages[ __shinken_test_type ].push( new OBJECT.Message( COMPONENT.CONST.STATUS.INFO, "[" + __shinken_test_type + "] - " + text_message ) );
     };
-    self.MakeFinalAction             = function ( timeout ) {
+    self.MakeFinalAction          = function ( timeout ) {
         setTimeout( function () {
-            self.buildMessage(0);
+            self.buildMessage( 0 );
             if ( !__is_single ) {
+                //if ( !SHINKEN.TOOLS.TEST_AUTO.hasMessage() ) {
                 SHINKEN.TOOLS.TEST_AUTO_PHASE.doNext();
+                //}
             }
         }, timeout );
     };
     self.buildMessage             = function ( timeout ) {
+        var phase = parseInt( SHINKEN.HIGHWAY.getParams( 'phase' ) || 0 );
         setTimeout( function () {
             SHINKEN.TOOLS.TEST_AUTO_LOGGER.showInConsole();
             if ( _has_script_error ) {
@@ -66,22 +69,22 @@ SHINKEN.TOOLS.TEST_AUTO       = (function ( self ) {
                 
                 switch ( _to_write.getStatus() ) {
                     case COMPONENT.CONST.STATUS.ERROR:
-                        SHINKEN.TOOLS.TEST_AUTO_LOGGER.ACTION.logToServer( __shinken_test_type, __phase, SHINKEN.TOOLS.TEST_AUTO.CONST.STATUS.ERROR, _to_write.buildMessage() );
+                        SHINKEN.TOOLS.TEST_AUTO_LOGGER.ACTION.logToServer( __shinken_test_type, phase, SHINKEN.TOOLS.TEST_AUTO.CONST.STATUS.ERROR, _to_write.buildMessage() );
                         break;
                     case COMPONENT.CONST.STATUS.WARNING:
-                        SHINKEN.TOOLS.TEST_AUTO_LOGGER.ACTION.logToServer( __shinken_test_type, __phase, SHINKEN.TOOLS.TEST_AUTO.CONST.STATUS.WARNING, _to_write.buildMessage() );
+                        SHINKEN.TOOLS.TEST_AUTO_LOGGER.ACTION.logToServer( __shinken_test_type, phase, SHINKEN.TOOLS.TEST_AUTO.CONST.STATUS.WARNING, _to_write.buildMessage() );
                         break;
                     case COMPONENT.CONST.STATUS.BEWARE:
-                        SHINKEN.TOOLS.TEST_AUTO_LOGGER.ACTION.logToServer( __shinken_test_type, __phase, SHINKEN.TOOLS.TEST_AUTO.CONST.STATUS.BEWARE, _to_write.buildMessage() );
+                        SHINKEN.TOOLS.TEST_AUTO_LOGGER.ACTION.logToServer( __shinken_test_type, phase, SHINKEN.TOOLS.TEST_AUTO.CONST.STATUS.BEWARE, _to_write.buildMessage() );
                         break;
                     default:
-                        SHINKEN.TOOLS.TEST_AUTO_LOGGER.ACTION.logToServer( __shinken_test_type, __phase, SHINKEN.TOOLS.TEST_AUTO.CONST.STATUS.INFO, _to_write.buildMessage() );
+                        SHINKEN.TOOLS.TEST_AUTO_LOGGER.ACTION.logToServer( __shinken_test_type, phase, SHINKEN.TOOLS.TEST_AUTO.CONST.STATUS.INFO, _to_write.buildMessage() );
                         break;
                 }
                 
             }
             else if ( !__is_single ) {
-                SHINKEN.TOOLS.TEST_AUTO_LOGGER.ACTION.logToServer( __shinken_test_type, __phase, SHINKEN.TOOLS.TEST_AUTO.CONST.STATUS.VALID, "" );
+                SHINKEN.TOOLS.TEST_AUTO_LOGGER.ACTION.logToServer( __shinken_test_type, phase, SHINKEN.TOOLS.TEST_AUTO.CONST.STATUS.VALID, "" );
                 console.log( "[TEST_AUTO DONE]", __shinken_test_type, SHINKEN.TOOLS.TEST_AUTO.CONST.STATUS.VALID, "__do_next = " + __do_next, "phase" + __phase );
             }
             else {
@@ -482,7 +485,7 @@ SHINKEN.TOOLS.TEST_AUTO_PHASE = (function ( self ) {
             var _test = eval( function_to_eval );
             if ( _test ) {
                 __do_next = true;
-                self.doNext( 0, __shinken_test_type, parseInt( __phase + 1 ) ); //TEST 00142
+                self.doNext( 0, __shinken_test_type, parseInt( __phase ) ); //TEST 00142
             }
             else {
                 self.doNextWhen( 1000, function_to_eval );
@@ -974,19 +977,25 @@ SHINKEN.TOOLS.TEST_AUTO_XSS = (function ( self ) {
 })( SHINKEN.TOOLS.TEST_AUTO_XSS || {} );
 
 SHINKEN.TOOLS.TEST_AUTO_OPEN_WINDOW = (function ( self ) {
-    self.DOM           = {};
-    self.TEST          = {};
-    self.ACTION        = {};
-    self.instance      = null;
+    self.DOM                      = {};
+    self.TEST                     = {};
+    self.ACTION                   = {};
+    self.instance                 = false;
     //********************************************  TEST   **************************************************//
-    self.TEST.pathName = function ( timeout, path_name ) {
+    self.TEST.pathName            = function ( timeout, path_name ) {
         setTimeout( function () {
             SHINKEN.TOOLS.TEST_AUTO.__isTextEqualTo( path_name, self.instance.location.pathname, "URL WINDOW" );
         }, timeout );
         return 200;
     };
+    self.TEST.redirect            = function ( timeout, path_name, path_expected ) {
+        setTimeout( function () {
+            SHINKEN.TOOLS.TEST_AUTO.__isTextEqualTo( path_name, path_expected, "URL REDIRECT" );
+        }, timeout );
+        return 200;
+    };
     //********************************************  ACTION   **************************************************//
-    self.ACTION.close  = function ( timeout ) {
+    self.ACTION.close             = function ( timeout ) {
         setTimeout( function () {
             if ( self.instance ) {
                 self.instance.close();
@@ -994,11 +1003,24 @@ SHINKEN.TOOLS.TEST_AUTO_OPEN_WINDOW = (function ( self ) {
         }, timeout );
         return 200;
     };
-    self.ACTION.open   = function ( timeout, url ) {
+    self.ACTION.open              = function ( timeout, url ) {
         setTimeout( function () {
             self.instance = window.open( url, "test", "width=" + window.innerWidth / 1.2 + ",height=" + window.innerHeight / 1.2 );
         }, timeout );
-        return 3000;
+        return 5000;
+    };
+    self.ACTION.openFull          = function ( timeout, url ) {
+        setTimeout( function () {
+            self.instance = window.open( url, "test", "width=1920,height=1080" );
+        }, timeout );
+        return 5000;
+    };
+    self.ACTION.openWithStructure = function ( timeout, phase_window ) {
+        var url = "http://" + SHINKEN.HIGHWAY.getHostName() + ":" + SHINKEN.TOOLS.TEST_AUTO.STRUCTURE[ __shinken_test_type ].port_window[ phase_window ] +
+                  SHINKEN.TOOLS.TEST_AUTO.STRUCTURE[ __shinken_test_type ].address_window[ phase_window ];
+        url += "?shinken_test_auto=" + __shinken_test_type + "&window_phase=" + (phase_window + 1);
+        
+        return self.ACTION.open( timeout, url );
     };
     return self;
 })( SHINKEN.TOOLS.TEST_AUTO_OPEN_WINDOW || {} );
\ No newline at end of file
