@@ -117,6 +117,12 @@ class DataHubMetaDriverReplication(AbstractDataHubDriver):
             driver.destroy()
     
     
+    def stop(self):
+        # type: () -> None
+        for driver in self.drivers:
+            driver.stop()
+    
+    
     def check_right_access_issues_then_fix_them(self):
         # type: () -> None
         for driver in self.drivers:
