@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2013-2019
+# Copyright (C) 2013-2022
 # This file is part of Shinken Enterprise, all rights reserved.
 
 import datetime
@@ -76,7 +76,7 @@ class ParseOptionError(SyntaxError):
     pass
 
 
-class NotFoundException(BaseException):
+class NotFoundException(Exception):
     pass
 
 
@@ -257,7 +257,7 @@ class Result(object):
     def exit(self, sorted_by_level=True):
         global to_print
         
-        if not self.titles:
+        if self.titles:
             self.titles.append(BREAK_LINE)
         if sorted_by_level:
             warnings = ''
@@ -309,7 +309,7 @@ class Result(object):
 
 class HTMLTag(object):
     used = False
-    STYLE = u'''.skn-ctg{margin:1px;background:#DDD;padding:1px 7px;border-radius:5px;box-sizing:border-box;display:inline-block;border:1px solid} .skn-brd{margin:7px 2px;padding:5px 7px;border-radius:7px;box-sizing:border-box;border:2px solid;width:99%;word-break: break-word;}'''
+    STYLE = u'''.skn-ctg{margin:1px;background:#DDD;padding:1px 7px;border-radius:5px;box-sizing:border-box;display:inline-block;border:1px solid} .skn-brd{margin:7px 2px;padding:5px 7px;border-radius:7px;box-sizing:border-box;border:2px solid;width:99%;word-break: break-word;border-collapse: separate;}'''
     EXTRA_STYLE = u''
     EXTRA_CLASS = u''
     
@@ -337,10 +337,10 @@ class HTMLTag(object):
     
     
     @staticmethod
-    def color_text(value, color=COLOR.BLACK, bold=True, italic=False):
-        # type: (unicode, unicode, bool, bool) -> unicode
-        _span = u'<span style="color:%(color)s;%(bold)s%(italic)s">%(value)s</span>'
-        _info = {u'color': color, u'value': value, u'bold': u'', u'italic': u''}
+    def color_text(value, color=COLOR.BLACK, bold=True, italic=False, extra_style=u''):
+        # type: (unicode, unicode, bool, bool, unicode) -> unicode
+        _span = u'<span style="color:%(color)s;%(bold)s%(italic)s%(extra_style)s">%(value)s</span>'
+        _info = {u'color': color, u'value': value, u'bold': u'', u'italic': u'', u'extra_style': extra_style}
         if bold:
             _info[u'bold'] = u'font-weight:bold;'
         if italic:
@@ -403,6 +403,17 @@ class HTMLTag(object):
             return HTMLTag.tag_value('normal', COLOR.BLACK)
 
 
+class HTMLComment(object):
+    used = False
+    STYLE = u'''.skn-com{color:grey;font-size:0.8em;text-indent:2em;display:inline-block;}'''
+    EXTRA_STYLE = u''
+    
+    @staticmethod
+    def add_tag(value):
+        HTMLComment.used = True
+        return u'<div class="skn-com";>%s</div>' % value
+
+
 class HTMLList(object):
     used = False
     STYLE = u'.skn-ul{padding: 0; margin: 0 0 10px 25px;} .skn-ul-compact{margin-bottom: 0px;padding-left: 0px;margin-left: 25px;}'
@@ -415,29 +426,36 @@ class HTMLList(object):
     
     
     @staticmethod
-    def _list_header(header, special_class=''):
-        if special_class != '':
+    def _list_header(header, special_class=u'', ordered_list=False):
+        list_type = u'ul'
+        if ordered_list:
+            list_type = u'ol'
+        if special_class != u'':
             HTMLList.used = True
         if header is None:
-            return u"<ul class='skn-ul %s'>" % special_class
+            return u"<%s class='skn-ul %s'>" % (list_type, special_class)
         else:
-            return "%s&nbsp;:<ul class='skn-ul %s'>" % (header, special_class)
+            return u"%s&nbsp;<%s class='skn-ul %s'>" % (header, list_type, special_class)
     
     
     @staticmethod
-    def _list_footer():
-        return u"</ul>"
+    def _list_footer(ordered_list=False):
+        if not ordered_list:
+            return u"</ul>"
+        else:
+            return u"</ol>"
     
     
     @staticmethod
-    def header_list(header, items, compact=False):
-        # type: (Optional[unicode], list, bool) -> unicode
-        special_class = 'skn-ul-compact' if compact else ''
+    def header_list(header, items, compact=False, ordered_list=False):
+        # type: (Optional[unicode], list, bool, bool) -> unicode
+        special_class = u'skn-ul-compact' if compact else u''
         HTMLList.used = True
         return u'%(header)s%(items)s%(footer)s' % {
-            'header': HTMLList._list_header(header, special_class=special_class),
-            'items' : "".join(map(lambda item: HTMLList._list_item(item), items)),
-            'footer': HTMLList._list_footer()}
+            u'header': HTMLList._list_header(header, special_class=special_class, ordered_list=ordered_list),
+            u'items' : u''.join(map(lambda item: HTMLList._list_item(item), items)),
+            u'footer': HTMLList._list_footer(ordered_list)
+        }
     
     
     @staticmethod
@@ -448,9 +466,9 @@ class HTMLList(object):
     # Just give me one bullet list
     @staticmethod
     def one_bullet_list(s):
-        # type: (Optional[str, unicode]) -> unicode
+        # type: (Optional[unicode]) -> unicode
         HTMLList.used = True  # Let the CSS be inserted
-        return u'%s%s%s' % (HTMLList._list_header(None, special_class='skn-ul-compact'), HTMLList._list_item(s), HTMLList._list_footer())
+        return u'%s%s%s' % (HTMLList._list_header(None, special_class=u'skn-ul-compact'), HTMLList._list_item(s), HTMLList._list_footer())
 
 
 class HTMLTable(object):
@@ -584,12 +602,14 @@ class Utils(object):
     
     @staticmethod
     def print_time(_time):
+        if not _time:
+            return u'No time'
         return datetime.datetime.fromtimestamp(_time).strftime('%x %X')
     
     
     @staticmethod
     def add_style(output):
-        html_items = (HTMLList, HTMLTable, HTMLTag)
+        html_items = (HTMLList, HTMLTable, HTMLTag, HTMLComment)
         
         active_item = []
         for html_item in html_items:
@@ -678,7 +698,7 @@ class ArbiterTraceVerificationTree(object):
                     # DEBUG: uncomment this line to simulate the in_architecture_names_conflict behavior
                     # architecture_names_conflicts_uuids[arbiter_architecture].append(master_arbiter_uuid)
         
-        # Maybe two distinct atchitecture (identified by the Arbiter MASTER server) have the same name: will be a problem in detection
+        # Maybe two distinct architecture (identified by the Arbiter MASTER server) have the same name: will be a problem in detection
         for (architecture_name, _uuids) in architecture_names_conflicts_uuids.items():
             if len(_uuids) >= 2:
                 in_architecture_names_conflict = True
@@ -722,7 +742,7 @@ class ArbiterTraceVerificationTree(object):
         
         for (master_arbiter_uuid, arbiters) in master_arbiter_uuids.items():
             
-            tree_entry = {u'architecture_name': u'', u'arbiters': []}
+            tree_entry = {u'architecture_name': u'', u'arbiters': [], u'ip': u''}
             self._arbiters_tree[master_arbiter_uuid] = tree_entry
             
             architecture_name = ''
@@ -730,6 +750,10 @@ class ArbiterTraceVerificationTree(object):
             # an error or not based on the number master_arbiter_uuids
             for arbiter in arbiters:
                 name = arbiter[u'name']
+                ip = arbiter.get(u'ip', u'')
+                if ip:
+                    tree_entry[u'ip'] = ip
+                
                 arbiter_architecture = arbiter.get(u'architecture_name', '')
                 if arbiter_architecture:
                     architecture_name = arbiter_architecture
@@ -738,7 +762,7 @@ class ArbiterTraceVerificationTree(object):
                 diff_time_with_arbiter = abs(arbiter[u'diff_time_with_arbiter'])  # abs: never believe a diff time
                 expire_period = arbiter[u'expire_period']
                 expire_time = insert_time + expire_period
-                # NOTE: take time from the server so we NEVER use the check local time (that can be wrong), and only the
+                # NOTE: take time from the server, so we NEVER use the check local time (that can be wrong), and only the
                 #       distant daemon state, even if WE are out of time
                 # this entry was missing before v02.07.06-Patched-08 (may 2021)
                 server_now = arbiter.get(u'now', int(time.time()))
@@ -756,7 +780,7 @@ class ArbiterTraceVerificationTree(object):
                     if worse_state == EXIT_STATUS.OK:  # do not change if was CRITICAL
                         worse_state = EXIT_STATUS.WARNING
                 else:
-                    _out = u'%s' % (last_connection_str)
+                    _out = u'%s' % last_connection_str
                     tree_entry[u'arbiters'].append((name, _out, EXIT_STATUS.OK))
             
             if not architecture_name:
@@ -871,28 +895,35 @@ class ShinkenUtils(object):
         in_conflict = verification_tree.get_in_conflict()
         in_architecture_names_conflict = verification_tree.get_in_architecture_names_conflict()
         state = verification_tree.get_state()
-        
+        arbiters_tree_sorted = sorted(arbiters_tree.items(), key=lambda item: (item[1][u'architecture_name'], item[1][u'ip']))
         master_arbiter_uuid_outputs = []
-        for master_arbiter_uuid, tree_entry in arbiters_tree.items():
-            architecture_name = tree_entry['architecture_name']
+        for master_arbiter_uuid, tree_entry in arbiters_tree_sorted:
+            architecture_name = tree_entry[u'architecture_name']
             
             arbiter_outputs = []
-            for arbiter_entry in tree_entry['arbiters']:
+            for arbiter_entry in tree_entry[u'arbiters']:
                 # 0: name, 1: txt, 2: status, 3: arbiter status
-                prefix = '%s ' % HTMLTag.tag_value(u'ERROR', COLOR.RED) if arbiter_entry[2] != EXIT_STATUS.OK else ''
-                arbiter_outputs.append(u'%s%s => <span style="color:purple">%s</span>' % (prefix, arbiter_entry[0], arbiter_entry[1]))
-            if in_architecture_names_conflict:  # If the name is not uniq, help the user to find it
-                arbiter_outputs.append(u'defined by the arbiter MASTER on the server with uuid %s' % master_arbiter_uuid)
-            architecture_header = u'Architecture <i>%s</i>' % architecture_name
-            master_arbiter_ouput = HTMLList.header_list(architecture_header, arbiter_outputs) + '<br/>'
-            master_arbiter_uuid_outputs.append(master_arbiter_ouput)
+                prefix = u'%s ' % HTMLTag.tag_value(u'ERROR', COLOR.RED) if arbiter_entry[2] != EXIT_STATUS.OK else u''
+                message = u'%s%s : <span style="color:purple">%s.</span>' % (prefix, arbiter_entry[0], arbiter_entry[1])
+                if in_conflict:
+                    comment = HTMLComment.add_tag(u'Defined on the server with uuid %s (/var/lib/shinken/server.uuid)' % master_arbiter_uuid)
+                    message = u'%s %s' % (message, comment)
+                arbiter_outputs.append(message)
+            architecture_header = u'<b>%s</b>  [<span style="color:purple">%s</span>] ' % (architecture_name, tree_entry[u'ip'])
+            master_arbiter_output = u'%s<br/>' % HTMLList.header_list(architecture_header, arbiter_outputs, compact=True)
+            master_arbiter_uuid_outputs.append(master_arbiter_output)
         
-        title = u'Arbiters connection:' if not in_conflict else HTMLTag.tag_value(u'Arbiters CONFLICT ( contacted by Arbiters from different architectures )', COLOR.RED)
-        output = HTMLList.header_list(title, master_arbiter_uuid_outputs)
+        if in_conflict:
+            title = HTMLTag.tag_value(u'<b>Arbiters CONFLICT</b>', COLOR.RED)
+        else:
+            title = u'Arbiters connection:'
+        
+        architecture_list = HTMLList.header_list(u'<p>Architecture List :</p>', master_arbiter_uuid_outputs, ordered_list=True)
+        output = u'%s<BLOCKQUOTE style="font-size:14px">%s</BLOCKQUOTE>' % (title, architecture_list)
         
         # We have architecture_name conflict, we need to help the user about how to find the servers
         if in_architecture_names_conflict:
-            output += u'NOTE:<i><ul><li>Some Arbiter MASTER on different servers have the same architecture name in their architecture_export module.</li><li>Look at "/var/lib/shinken/server.uuid" to identify them</li></ul></i>'
+            output += u'NOTE: <i><ul>Some architecture have the same name. We advise you to change it in the configuration of their architecture_export module.</i></ul>'
         
         # Only display architecture if there is some conflicts
         if state != EXIT_STATUS.OK:
@@ -925,6 +956,11 @@ class ShinkenUtils(object):
     
     @staticmethod
     def minimal_check(result, data, daemon_type, shinken_supervisor_version):
+        is_interrupted = data.get('is_interrupted', False)
+        if is_interrupted:
+            output = 'The %s is performing a shutdown.' % daemon_type
+            result.hard_exit(EXIT_STATUS.WARNING, output)
+        
         have_conf = data.get('have_conf', False)
         if not have_conf:
             output = 'No configuration given by an Arbiter for now.'
