@@ -76,8 +76,8 @@ def _split_node_graphite(def_node):
 
 
 class GraphiteConfReader(object):
-    def __init__(self, graphite_hostname, graphite_port, ssh_port, ssh_key_file, passphrase, graphite_conf_file, user):
-        # type: (unicode, unicode, unicode, int, unicode, unicode, unicode) -> None
+    def __init__(self, graphite_hostname, graphite_port, ssh_port, ssh_key_file, passphrase, graphite_conf_file, user, is_children_node=False):
+        # type: (unicode, unicode, unicode, int, unicode, unicode, unicode, bool) -> None
         self.result = Result()
         self.graphite_hostname = graphite_hostname
         self.graphite_port = unicode(graphite_port)
@@ -92,6 +92,7 @@ class GraphiteConfReader(object):
         self.is_relay = None
         self.client = None
         self.graphite_process_name = u''
+        self.is_children_node = is_children_node
     
     
     def connect(self, quit_on_fail=True, timeout=None):
