@@ -7,7 +7,7 @@
 import os
 import threading
 
-from ..portalocker import RLock, LOCK_EX
+from ..portalocker import RLock, LOCK_EX, LOCK_SH
 from ...os_helper import set_ownership
 
 
@@ -27,9 +27,10 @@ def create_tree(path, user='shinken', group='shinken', mode=0o755):
 
 
 class ShinkenInterProcessRLock(object):
-    def __init__(self, filename):
+    def __init__(self, filename, shared_lock=False):
         
         self._lock_file = filename
+        self._lock_mode = LOCK_SH if shared_lock else LOCK_EX
         self._locks = {}
         dirname = os.path.dirname(filename)
         create_tree(dirname)
@@ -38,6 +39,18 @@ class ShinkenInterProcessRLock(object):
             set_ownership(filename)
     
     
+    def destroy(self):
+        for thread_locks in self._locks.itervalues():
+            for other_lock in thread_locks.itervalues():
+                if other_lock and other_lock.fh and not other_lock.fh.closed:
+                    other_lock.fh.close()
+        self._locks = {}
+        try:
+            os.unlink(self._lock_file)
+        except:
+            pass
+    
+    
     def _get_lock(self):
         # type: () -> RLock
         pid = os.getpid()
@@ -50,7 +63,7 @@ class ShinkenInterProcessRLock(object):
             self._locks = {pid: {}}
         thread_id = threading.current_thread().ident
         if thread_id not in self._locks[pid]:
-            self._locks[pid][thread_id] = RLock(self._lock_file, timeout=None, mode='a+b', flags=LOCK_EX)
+            self._locks[pid][thread_id] = RLock(self._lock_file, timeout=None, mode='a+b', flags=self._lock_mode)
         
         return self._locks[pid][thread_id]
     
